/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.entity;

import java.io.Serializable;
import p2pMapReduce.mapreduceModule.TaskType;
import p2pMapReduce.entity.Task.TaskStatus;

/**
 * Task view for a primary or backup master.
 * 
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class JobTask implements Serializable {

    /**
     * The associated task
     */
    private Task task;
    /**
     * The id of the slave node on which this task has been submitted
     */
    private String slaveID;
    /**
     * 
     */
    private Task.TaskStatus taskStatus;
    /**
     * Only for successfully reduce tasks.
     * Indicates if the user node has retrieved the output
     * of this task.
     */
    private boolean outputRetrieved;

    /**
     * For searching purpose only
     * 
     * @param task
     */
    public JobTask(Task task) {
        this.task = task;
    }

    public JobTask(Task task, String slaveID) {
        this(task, slaveID, TaskStatus.ASSIGNED);
    }

    public JobTask(Task task, String slaveID, TaskStatus taskStatus) {
        this.task = task;
        this.slaveID = slaveID;
        this.taskStatus = taskStatus;
    }

    public String getSlaveID() {
        return slaveID;
    }

    public void setSlaveID(String slaveID) {
        this.slaveID = slaveID;
    }

    public String getTaskAttIDAsString() {
        return task.getTaskAttIDAsString();
    }

    public Task getTask() {
        return task;
    }

    public void setTask(Task task) {
        this.task = task;
    }

    public TaskStatus getTaskStatus() {
        return taskStatus;
    }

    public void setTaskStatus(TaskStatus taskStatus) {
        this.taskStatus = taskStatus;
    }

    public boolean isOutputRetrieved() {
        return outputRetrieved;
    }

    public void setOutputRetrieved(boolean outputRetrieved) {
        this.outputRetrieved = outputRetrieved;
    }

    public boolean isMap() {
        return getTask().getTaskType() == TaskType.MAP;
    }

    public boolean isReduce() {
        return getTask().getTaskType() == TaskType.REDUCE;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final JobTask other = (JobTask) obj;
        if (this.task != other.task && (this.task == null || !this.task.equals(other.task))) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 53 * hash + (this.task != null ? this.task.hashCode() : 0);
        return hash;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("JobTask [");
        sb.append("taskID= ").append(task.getTaskAttIDAsString());
        sb.append(" taskStatus=").append(taskStatus);
        sb.append(" slaveID=").append(slaveID);
        sb.append("]");
        return sb.toString();
    }
}
