/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.entity;

import java.io.Serializable;
import java.util.LinkedList;
import p2pMapReduce.mapreduceModule.Job;
import p2pMapReduce.mapreduceModule.JobStatus;
import p2pMapReduce.networkModule.NodeInfo;
import p2pMapReduce.util.LoggingUtil;



/**
 * The primary node view of a job.
 * 
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class PrimaryJob implements Serializable {

    /**
     * The associated job
     */
    private Job job;
    /**
     * To provide the direct access to node information of user node
     * without searching for NodeInfo in the network
     */
    private NodeInfo userNodeInfo;
    /**
     * The list of all task attempt submitted for this job.
     */
    private LinkedList<JobTask> jobTaskList;
    /**
     * List of all the node that acts as backup for this job
     */
    private LinkedList<String> backupMasterList;

    public PrimaryJob(Job job, NodeInfo userNodeInfo) {
        this.job = job;
        this.userNodeInfo = userNodeInfo;
        jobTaskList = new LinkedList<JobTask>();
        backupMasterList = new LinkedList<String>();

    }

    public PrimaryJob(BackupJob backupJob) {
        this.job = backupJob.getJob();
        this.userNodeInfo = backupJob.getUserNodeInfo();
        this.jobTaskList = backupJob.getJobTaskList();
        this.backupMasterList = backupJob.getBackupMasterList();
    }

    public String getJobID() {
        return job.getJobID().toString();
    }

    public LinkedList<String> getBackupMasterList() {
        return backupMasterList;
    }

    public void setBackupMasterList(LinkedList<String> backupMasterList) {
        this.backupMasterList = backupMasterList;
    }

    public Job getJob() {
        return job;
    }

    public void setJob(Job job) {
        this.job = job;
    }

    public JobStatus.State getJobState() {
        return job.getJobState();
    }

    public void setJobState(JobStatus.State jobState) {
        job.setJobState(jobState);
    }

    public LinkedList<JobTask> getJobTaskList() {
        return jobTaskList;
    }

    public void setJobTaskList(LinkedList<JobTask> jobTaskList) {
        this.jobTaskList = jobTaskList;
    }

    public NodeInfo getUserNodeInfo() {
        return userNodeInfo;
    }

    public void setUserNodeInfo(NodeInfo userNodeInfo) {
        this.userNodeInfo = userNodeInfo;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("PrimaryJob [jobID:").append(job.getJobID()).append(" jobState:").append(job.getJobState()).append("  user:").append(userNodeInfo.getID()).append("\n");
        sb.append("JobTaskList=\n").append(LoggingUtil.elementByLine(jobTaskList));
        sb.append("BackupMasterList=\n").append(LoggingUtil.elementByLine(backupMasterList));
        sb.append("\n]");
        return sb.toString();
    }
}
