/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.entity;

import java.io.Serializable;
import p2pMapReduce.mapreduceModule.conf.Configuration;
import p2pMapReduce.nodeModule.NodeModuleIF;
import p2pMapReduce.mapreduceModule.TaskAttemptID;
import p2pMapReduce.mapreduceModule.TaskID;
import p2pMapReduce.mapreduceModule.TaskType;

/**
 * Base class for Map and Reduce tasks
 * 
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public abstract class Task implements Serializable {

    public enum TaskStatus {

        NOT_ASSIGNED,
        ASSIGNED,
        SUCCESS,
        FAILED
    };
    Configuration conf;
    TaskAttemptID taskAttID;
    /**
     * Represents the number of map or reduce task for which this task is
     * an attempt
     */
    int partition;

    /**
     * For searching purpose only
     * @param taskAttID
     */
    public Task(String taskAttID) {
        this.taskAttID = TaskAttemptID.forName(taskAttID);
    }

    public Task(Configuration conf, TaskAttemptID taskAttID) {
        this.conf = conf;
        this.taskAttID = taskAttID;
        this.partition = taskAttID.getTaskID().getId();

    }

    /**
     *
     * @param slaveTask
     * @param nodeModule
     * @return
     */
    public abstract TaskRunner createTaskRunner(SlaveTask slaveTask, NodeModuleIF nodeModule);

    public Configuration getConf() {
        return conf;
    }

    public void setConf(Configuration conf) {
        this.conf = conf;
    }

    public TaskID getTaskID() {
        return taskAttID.getTaskID();
    }

    public TaskAttemptID getTaskAttID() {
        return taskAttID;
    }

    public void setTaskAttID(TaskAttemptID taskAttID) {
        this.taskAttID = taskAttID;
    }

    public String getTaskAttIDAsString() {
        return taskAttID.toString();
    }

    public void setTaskAttID(String taskID) {
        taskAttID = TaskAttemptID.forName(taskID);
    }

    public String getJobID() {
        return taskAttID.getJobID().toString();
    }

    public TaskType getTaskType() {
        return taskAttID.getTaskType();
    }

    public int getPartition() {
        return partition;
    }

    public void setPartition(int partition) {
        this.partition = partition;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Task other = (Task) obj;
        if (this.taskAttID != other.taskAttID && (this.taskAttID == null || !this.taskAttID.equals(other.taskAttID))) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 23 * hash + (this.taskAttID != null ? this.taskAttID.hashCode() : 0);
        return hash;
    }

    @Override
    public String toString() {
        return "Task [taskID=" + taskAttID + "]";
    }
}
