/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.entity;

import java.io.File;
import java.net.URL;
import java.util.concurrent.Callable;
import p2pMapReduce.mapreduceModule.TaskAttemptContext;
import p2pMapReduce.nodeModule.NodeModuleIF;
import p2pMapReduce.util.ClassLoaderUtil;

/**
 * Base class for task runners.
 * The class implements the {@link Callable} interface in order to provide
 * the task completion status as the task execution result.
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public abstract class TaskRunner implements Callable<Task.TaskStatus> {

    /**
     * A reference to the {@link SlaveTask} associate to this
     * TaskRunner.
     * It is needed by map tasks for input retrieving.
     */
    SlaveTask slaveTask;
    TaskAttemptContext taskAttContext;
    NodeModuleIF nodeModule;

    public TaskRunner(SlaveTask task, NodeModuleIF nodeModule) {
        this.slaveTask = task;
        this.nodeModule = nodeModule;
    }

    protected void updateClassPath(String jarsDirPath) throws Exception {
        File jarsDir = new File(jarsDirPath);
        File jarsFiles[] = jarsDir.listFiles();
        URL jarsURLs[] = new URL[jarsFiles.length];
        for (int i = 0; i < jarsFiles.length; i++) {
            File file = jarsFiles[i];
            jarsURLs[i] = file.toURI().toURL();
        }

        ClassLoaderUtil.addToSystemClassLoader(jarsURLs);

    }
}
