/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.fs;

import java.io.File;
import java.util.Random;

/**
 * Gives a file system view pointed to a specified root directory
 * 
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class FileSystemView {

    String rootDir;

    public FileSystemView(String rootDir) throws IllegalArgumentException {
        this.rootDir = rootDir;
        if (new File(rootDir).isFile()) {
            throw new IllegalArgumentException("Root must be a directory");
        }
    }

    public String getRootDir() {
        return rootDir;
    }

    public String createPath(String path) {
        return rootDir + "/" + path;
    }

    public File createFile(String path) {
        return new File(rootDir, path);
    }

    public boolean mkdir(String path) {
        File f = new File(rootDir, path);
        return f.mkdir();
    }

    public boolean mkdirs(String path) {
        File f = new File(rootDir, path);
        return f.mkdirs();
    }

    public boolean delete(String filePath) {
        File f = createFile(filePath);
        return f.delete();
    }

    /**
     * Creates a random named directory in the ftp home dir.
     * Returns the path of the created directory relative to the
     * ftp home dir
     *
     * @return a random directory name
     */
    public String createTempDir() {
        Random r = new Random();
        String dir = "tmp_" + r.nextInt() + "/";
        File f = createFile(dir);
        while (!f.mkdirs()) {
            dir = "tmp_" + r.nextInt() + "/";
            f = createFile(dir);
        }

        return dir;
    }
}
