/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.ftp;

import java.io.IOException;
import java.net.InetAddress;
import p2pMapReduce.networkModule.NodeInfo;

/**
 * A component charged with ftp server and client functionality.
 * An instance could be retrieved through
 * {@link FTPManagerFactory#createFTPManager() }
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public interface FTPManager {

    /**
     * Starts the local ftp server
     *
     * @throws IOException
     */
    void startLocalServer() throws IOException;

    /**
     * Returns the path to the directory used as root directory by FTP server
     *
     * @return The path to FTP server root directory, {@code null} if the
     * server has not been stared yet
     */
    String getBaseDir();

    /**
     * The local port on which FTP server listen
     *
     * @return
     */
    int getFtpPort();

    /**
     * Find for a {@link InetAddress} which is not a loopback address
     * and return {@link InetAddress#getHostAddress()} result.
     * If all {@link InetAddress} are loopback addresses utilize
     * {@link InetAddress#getLocalHost() }
     *
     * @return The string representation of the ip address for this FTP server
     */
    String getIpAddress();

    /**
     * Download the content of the directory {@code dirPath} from
     * the specified node in a directory with the same path on the local ftp
     * base directory
     *
     * @param remoteNode
     * @param dirPath
     * @return
     */
    boolean downloadDir(NodeInfo remoteNode, String dirPath);

    /**
     * Download the content of the directory {@code dirPath} from
     * the specified node in the directory {@code destDir}
     *
     *
     * @param remoteNode
     * @param dirPath
     * @param destDir
     * @return True if the operation is successfully completed
     */
    boolean downloadDir(NodeInfo remoteNode, String dirPath, String destDir);

    /**
     * Download the specified file from the specified node.
     * The file will be downloaded in the same path in the local ftp base
     * directory
     *
     * @param remoteNode
     * @param filePath
     * @return
     */
    boolean downloadFile(NodeInfo remoteNode, String filePath);

    /**
     * Download the file {@code srcFilePath} from the specified node.
     * The file will be downloaded in the specified path {@code dstFilePath}
     * which must contain also the file name.
     *
     * @param remoteNode
     * @param srcFilePath
     * @param dstFilePath
     * @return
     */
    boolean downloadFile(NodeInfo remoteNode, String srcFilePath, String dstFilePath);
}
