/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.ftp;

import p2pMapReduce.impl.ftp.FTPManagerImpl;
import java.io.File;
import java.io.IOException;
import java.net.ServerSocket;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Factory for {@link FTPManager } creation.
 * 
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class FTPManagerFactory {

    private static final transient Logger LOG = Logger.getLogger(FTPManagerFactory.class.getName());
    public static final String FTP_USERS_FILE = "conf/users.properties";

    public static FTPManager createFTPManager() {
        File f = new File(FTP_USERS_FILE);
        int port = findFreeLocalPort();
        if (port == -1) {
            return null;
        }
        return new FTPManagerImpl(f, port);
    }

    private static int findFreeLocalPort() {
        ServerSocket ss = null;
        int port = -1;
        try {
            ss = new ServerSocket(0);
            ss.setReuseAddress(true);
            port = ss.getLocalPort();
        } catch (IOException ex) {
            LOG.log(Level.SEVERE, null, ex);
        } finally {
            if (ss != null) {
                try {
                    ss.close();
                } catch (IOException ex) {
                    LOG.log(Level.SEVERE, null, ex);
                }
            }
        }

        return port;
    }

    private FTPManagerFactory() {
    }
}
