/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.gui;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.LinkedList;
import java.util.logging.Logger;
import javax.swing.JFrame;
import p2pMapReduce.entity.BackupJob;
import p2pMapReduce.entity.JobTask;
import p2pMapReduce.mapreduceModule.TaskID;
import p2pMapReduce.nodeModule.nodeFsm.NodeFSM;

/**
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class BackupJobViewer extends javax.swing.JPanel implements Updatable {

    private static final transient Logger LOG = Logger.getLogger(BackupJobViewer.class.getName());
    static NodeIDRenderer nodeIDRenderer = IDRendererFactory.getNodeIDRenderer();

    public static void show(BackupJob backupJob, NodeFSM nodeFSM, GUIFrame guiFrame) {
        JobIDRenderer jobIDRenderer = IDRendererFactory.getJobIDRenderer();
        String title = "M ";//only for master
        title += nodeIDRenderer.render(nodeFSM.getNodeModule().getNodeID()) + " - ";
        title += "BackupJob " + jobIDRenderer.render(backupJob.getJob().getJobID());
        JFrame frame = new JFrame(title);
        frame.setIconImage(Resource.getInstance().getImageIcon(GUI_Constants.LOGO_IMAGE).getImage());
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        BackupJobViewer backupJobViewer = new BackupJobViewer(backupJob, nodeFSM, frame);
        frame.setContentPane(backupJobViewer);
        frame.pack();
        Locator.locateCenter(frame);
        frame.setVisible(true);
        guiFrame.getUpdater().addUpdatable(backupJobViewer);
    }
    BackupJob backupJob;
    NodeFSM nodeFSM;
    boolean noMoreUpdate;

    public BackupJobViewer(BackupJob primaryJob, NodeFSM nodeFSM, JFrame frame) {
        this.backupJob = primaryJob;
        this.nodeFSM = nodeFSM;
        initComponents();

        backupJobIDL.setText(backupJob.getJobID());
        frame.addWindowListener(new WindowAdapter() {

            @Override
            public void windowClosing(WindowEvent e) {
                noMoreUpdate = true;
            }

            @Override
            public void windowClosed(WindowEvent e) {
                noMoreUpdate = true;
            }
        });
    }

    public boolean noMoreUpdate() {
        return noMoreUpdate;
    }

    public void update() {

        StringBuilder sb = new StringBuilder();
        /*
         * in order to avoid concurrent modif exception during execution
         * 
         */
        LinkedList<String> backupMasters = new LinkedList<String>();
        synchronized (nodeFSM) {
            if (nodeFSM.getNodeModule().getBackupJob(backupJob.getJobID()) == null) {
                noMoreUpdate = false;
            }
            backupMasters.addAll(backupJob.getBackupMasterList());
        }
        for (String backupMasterID : backupMasters) {
            sb.append(nodeIDRenderer.render(backupMasterID)).append("\n");
        }
        backupMasters.clear();
        backupMasterTA.setText(sb.toString());

        sb = new StringBuilder();
        LinkedList<JobTask> jobTasks = new LinkedList<JobTask>();
        synchronized (nodeFSM) {

            jobTasks.addAll(backupJob.getJobTaskList());
        }
        int submitted = jobTasks.size();
        int runningTasks = 0;
        int failedTasks = 0;
        int successfulTasks = 0;
        for (JobTask jobTask : jobTasks) {
            String taskAttID = jobTask.getTask().getTaskAttIDAsString();
            char type = TaskID.getRepresentingCharacter(jobTask.getTask().getTaskType());
            int charIndex = taskAttID.lastIndexOf(type);

            sb.append(taskAttID.substring(charIndex)).append(" ");
            sb.append(nodeIDRenderer.render(jobTask.getSlaveID())).append(" ");
            sb.append(jobTask.getTaskStatus()).append(" ");
            sb.append("\n");

            switch (jobTask.getTaskStatus()) {
                case ASSIGNED:
                    runningTasks++;
                    break;
                case FAILED:
                    failedTasks++;
                    break;
                case SUCCESS:
                    successfulTasks++;
                    break;
                default:
                    throw new AssertionError();
            }

        }
        jobTasks.clear();
        jobTasksTA.setText(sb.toString());


        primaryIDL.setText(nodeIDRenderer.render(backupJob.getPrimaryID()));
        stateLabel.setText(backupJob.getJobState().toString());
        mapProgressBar.setValue((int) (backupJob.getJob().getStatus().getMapProgress() * 100));
        reduceProgressBar.setValue((int) (backupJob.getJob().getStatus().getReduceProgress() * 100));
        submittedTasksL.setText("" + submitted);
        runningTasksL.setText("" + runningTasks);
        failedTaskL.setText("" + failedTasks);
        successfullTaskL.setText("" + successfulTasks);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        jobTasksTA = new javax.swing.JTextArea();
        jScrollPane2 = new javax.swing.JScrollPane();
        backupMasterTA = new javax.swing.JTextArea();
        javax.swing.JLabel jLabel1 = new javax.swing.JLabel();
        javax.swing.JLabel jLabel2 = new javax.swing.JLabel();
        jPanel1 = new javax.swing.JPanel();
        javax.swing.JLabel jLabel3 = new javax.swing.JLabel();
        primaryIDL = new javax.swing.JLabel();
        javax.swing.JLabel jLabel4 = new javax.swing.JLabel();
        runningTasksL = new javax.swing.JLabel();
        javax.swing.JLabel jLabel6 = new javax.swing.JLabel();
        javax.swing.JLabel jLabel7 = new javax.swing.JLabel();
        failedTaskL = new javax.swing.JLabel();
        successfullTaskL = new javax.swing.JLabel();
        javax.swing.JLabel jLabel5 = new javax.swing.JLabel();
        submittedTasksL = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        backupJobIDL = new javax.swing.JLabel();
        javax.swing.JLabel jLabel9 = new javax.swing.JLabel();
        stateLabel = new javax.swing.JLabel();
        javax.swing.JLabel jLabel10 = new javax.swing.JLabel();
        mapProgressBar = new javax.swing.JProgressBar();
        javax.swing.JLabel jLabel11 = new javax.swing.JLabel();
        reduceProgressBar = new javax.swing.JProgressBar();

        jobTasksTA.setColumns(20);
        jobTasksTA.setEditable(false);
        jobTasksTA.setFont(new java.awt.Font("Dialog", 0, 13)); // NOI18N
        jobTasksTA.setRows(5);
        jScrollPane1.setViewportView(jobTasksTA);

        backupMasterTA.setColumns(20);
        backupMasterTA.setEditable(false);
        backupMasterTA.setFont(new java.awt.Font("Dialog", 0, 13)); // NOI18N
        backupMasterTA.setRows(5);
        jScrollPane2.setViewportView(backupMasterTA);

        jLabel1.setText("Backup masters:");

        jLabel2.setText("Tasks:");

        jLabel3.setText("Primary node ID:");

        primaryIDL.setFont(new java.awt.Font("Ubuntu", 0, 14));

        jLabel4.setText("Assigned tasks:");

        jLabel6.setText("Failed tasks:");

        jLabel7.setText("Successfull tasks:");

        jLabel5.setText("Submitted tasks:");

        jLabel8.setText("Backup job ID:");

        backupJobIDL.setFont(new java.awt.Font("Ubuntu", 0, 14));

        jLabel9.setText("Job state:");

        jLabel10.setText("Map phase");

        mapProgressBar.setFont(new java.awt.Font("Ubuntu", 1, 14)); // NOI18N
        mapProgressBar.setStringPainted(true);

        jLabel11.setText("Reduce phase");

        reduceProgressBar.setFont(new java.awt.Font("Ubuntu", 1, 14)); // NOI18N
        reduceProgressBar.setStringPainted(true);

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                        .addGroup(jPanel1Layout.createSequentialGroup()
                            .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addComponent(jLabel7)
                                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                                    .addComponent(jLabel6, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                    .addComponent(jLabel4, javax.swing.GroupLayout.Alignment.LEADING)))
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                            .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addComponent(failedTaskL, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 691, Short.MAX_VALUE)
                                .addComponent(runningTasksL, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 691, Short.MAX_VALUE)
                                .addComponent(successfullTaskL, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 691, Short.MAX_VALUE)))
                        .addGroup(jPanel1Layout.createSequentialGroup()
                            .addComponent(jLabel5)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                            .addComponent(submittedTasksL, javax.swing.GroupLayout.PREFERRED_SIZE, 691, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                        .addGroup(jPanel1Layout.createSequentialGroup()
                            .addComponent(jLabel3)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                            .addComponent(primaryIDL, javax.swing.GroupLayout.PREFERRED_SIZE, 691, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGroup(jPanel1Layout.createSequentialGroup()
                            .addComponent(jLabel8)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                            .addComponent(backupJobIDL, javax.swing.GroupLayout.DEFAULT_SIZE, 691, Short.MAX_VALUE)))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jLabel9)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(stateLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 147, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(jLabel10)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(mapProgressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(jLabel11)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(reduceProgressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel1Layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {jLabel3, jLabel4, jLabel5, jLabel6, jLabel7, jLabel8, jLabel9});

        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel8)
                    .addComponent(backupJobIDL))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(primaryIDL, javax.swing.GroupLayout.PREFERRED_SIZE, 18, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 3, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel9)
                            .addComponent(stateLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 18, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addComponent(jLabel10)
                    .addComponent(mapProgressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel11)
                    .addComponent(reduceProgressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel5)
                    .addComponent(submittedTasksL, javax.swing.GroupLayout.PREFERRED_SIZE, 18, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel4)
                    .addComponent(runningTasksL, javax.swing.GroupLayout.PREFERRED_SIZE, 18, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jLabel6)
                    .addComponent(failedTaskL, javax.swing.GroupLayout.PREFERRED_SIZE, 18, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel7)
                    .addComponent(successfullTaskL, javax.swing.GroupLayout.PREFERRED_SIZE, 18, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        jPanel1Layout.linkSize(javax.swing.SwingConstants.VERTICAL, new java.awt.Component[] {backupJobIDL, failedTaskL, jLabel3, jLabel4, jLabel5, jLabel6, jLabel7, primaryIDL, runningTasksL, submittedTasksL, successfullTaskL});

        jPanel1Layout.linkSize(javax.swing.SwingConstants.VERTICAL, new java.awt.Component[] {jLabel10, jLabel11, mapProgressBar, reduceProgressBar});

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 832, Short.MAX_VALUE)
                    .addComponent(jLabel2)
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 832, Short.MAX_VALUE)
                    .addComponent(jLabel1)
                    .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabel1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabel2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 268, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel backupJobIDL;
    private javax.swing.JTextArea backupMasterTA;
    private javax.swing.JLabel failedTaskL;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JTextArea jobTasksTA;
    private javax.swing.JProgressBar mapProgressBar;
    private javax.swing.JLabel primaryIDL;
    private javax.swing.JProgressBar reduceProgressBar;
    private javax.swing.JLabel runningTasksL;
    private javax.swing.JLabel stateLabel;
    private javax.swing.JLabel submittedTasksL;
    private javax.swing.JLabel successfullTaskL;
    // End of variables declaration//GEN-END:variables
}
