/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.DefaultListModel;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListCellRenderer;
import p2pMapReduce.entity.BackupJob;
import p2pMapReduce.entity.PrimaryJob;
import p2pMapReduce.entity.SlaveTask;
import p2pMapReduce.impl.networkModule.jxta.JxtaNetworkModule;
import p2pMapReduce.impl.nodeModule.NodeModuleImpl;
import p2pMapReduce.util.P2PMapReduceConfig;

/**
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class ExecutionPanel extends javax.swing.JPanel implements Updatable {

    private static final transient Logger LOG = Logger.getLogger(ExecutionPanel.class.getName());
    JxtaNetworkModule networkModule;
    NodeModuleImpl nodeModule;
    GUIFrame guiFrame;

    public ExecutionPanel(NodeModuleImpl nodeModule, GUIFrame guiFrame) {
        this.nodeModule = nodeModule;
        this.guiFrame = guiFrame;
        this.networkModule = (JxtaNetworkModule) nodeModule.getNetworkModule();

        initComponents();

        jobAsPrimaryJList.setCellRenderer(new PrimaryJobListCellRenderer());
        jobAsBackupJList.setCellRenderer(new BackupJobListCellRenderer());
        tasksAsSlaveJL.setCellRenderer(new TaskListCellRenderer());
        JobListMouseListener mouseList = new JobListMouseListener();
        jobAsPrimaryJList.addMouseListener(mouseList);
        jobAsBackupJList.addMouseListener(mouseList);
        TaskListMouseListener taskListMouseListener = new TaskListMouseListener();
        tasksAsSlaveJL.addMouseListener(taskListMouseListener);

    }

    public boolean noMoreUpdate() {
        return false;
    }

    public void update() {

        ArrayList<PrimaryJob> primaryJobs = new ArrayList<PrimaryJob>(nodeModule.getPrimaryJobList().size());
        synchronized (nodeModule.getNodeFSM()) {
            primaryJobs.addAll(nodeModule.getPrimaryJobList());
        }
        jobAsPrimaryNL.setText("" + primaryJobs.size());
        jobAsPrimaryJList.setModel(new PrimaryJobListModel(primaryJobs));

        ArrayList<BackupJob> backupJobs = new ArrayList<BackupJob>(nodeModule.getBackupJobList().size());

        synchronized (nodeModule.getNodeFSM()) {
            backupJobs.addAll(nodeModule.getBackupJobList());
        }
        jobsAsBackupNL.setText("" + backupJobs.size());
        jobAsBackupJList.setModel(new BackupJobListModel(backupJobs));

        ArrayList<SlaveTask> tasks = new ArrayList<SlaveTask>(nodeModule.getSlaveTaskMap().size());

        synchronized (nodeModule.getNodeFSM()) {
            tasks.addAll(nodeModule.getSlaveTaskMap().values());
        }
        taskNumberL.setText("" + tasks.size());
        tasksAsSlaveJL.setModel(new TaskListModel(tasks));

        repaint();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        headerPanel = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        javax.swing.JLabel jLabel1 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jobAsPrimaryJList = new javax.swing.JList();
        javax.swing.JLabel jLabel2 = new javax.swing.JLabel();
        jScrollPane2 = new javax.swing.JScrollPane();
        jobAsBackupJList = new javax.swing.JList();
        jLabel3 = new javax.swing.JLabel();
        jScrollPane3 = new javax.swing.JScrollPane();
        tasksAsSlaveJL = new javax.swing.JList();
        taskNumberL = new javax.swing.JLabel();
        jobAsPrimaryNL = new javax.swing.JLabel();
        jobsAsBackupNL = new javax.swing.JLabel();

        setPreferredSize(new java.awt.Dimension(747, 610));

        javax.swing.GroupLayout headerPanelLayout = new javax.swing.GroupLayout(headerPanel);
        headerPanel.setLayout(headerPanelLayout);
        headerPanelLayout.setHorizontalGroup(
            headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 727, Short.MAX_VALUE)
        );
        headerPanelLayout.setVerticalGroup(
            headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 56, Short.MAX_VALUE)
        );

        jLabel1.setText("Jobs as primary master:");

        jobAsPrimaryJList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jobAsPrimaryJList.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
        jScrollPane1.setViewportView(jobAsPrimaryJList);

        jLabel2.setText("Jobs as backup master:");

        jobAsBackupJList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jobAsBackupJList.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
        jScrollPane2.setViewportView(jobAsBackupJList);

        jLabel3.setText("Tasks as slave:");

        tasksAsSlaveJL.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        tasksAsSlaveJL.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
        jScrollPane3.setViewportView(tasksAsSlaveJL);

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addComponent(jLabel1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jobAsPrimaryNL, javax.swing.GroupLayout.PREFERRED_SIZE, 556, javax.swing.GroupLayout.PREFERRED_SIZE))
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 727, Short.MAX_VALUE)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addComponent(jLabel2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jobsAsBackupNL, javax.swing.GroupLayout.PREFERRED_SIZE, 556, javax.swing.GroupLayout.PREFERRED_SIZE))
            .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 727, Short.MAX_VALUE)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addComponent(jLabel3)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(taskNumberL, javax.swing.GroupLayout.PREFERRED_SIZE, 556, javax.swing.GroupLayout.PREFERRED_SIZE))
            .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 727, Short.MAX_VALUE)
        );

        jPanel1Layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {jLabel1, jLabel2, jLabel3});

        jPanel1Layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {jobAsPrimaryNL, jobsAsBackupNL, taskNumberL});

        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(jobAsPrimaryNL, javax.swing.GroupLayout.PREFERRED_SIZE, 18, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 122, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2)
                    .addComponent(jobsAsBackupNL, javax.swing.GroupLayout.PREFERRED_SIZE, 18, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane2, javax.swing.GroupLayout.PREFERRED_SIZE, 122, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(taskNumberL, javax.swing.GroupLayout.PREFERRED_SIZE, 18, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 178, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(headerPanel, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(headerPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel headerPanel;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JList jobAsBackupJList;
    private javax.swing.JList jobAsPrimaryJList;
    private javax.swing.JLabel jobAsPrimaryNL;
    private javax.swing.JLabel jobsAsBackupNL;
    private javax.swing.JLabel taskNumberL;
    private javax.swing.JList tasksAsSlaveJL;
    // End of variables declaration//GEN-END:variables

    private class PrimaryJobListModel extends DefaultListModel {

        ArrayList<PrimaryJob> primaryJobs;

        public PrimaryJobListModel(ArrayList<PrimaryJob> primaryJobs) {
            this.primaryJobs = primaryJobs;
        }

        @Override
        public Object getElementAt(int index) {
            return primaryJobs.get(index);
        }

        @Override
        public int getSize() {
            return primaryJobs.size();
        }
    }

    private class BackupJobListModel extends DefaultListModel {

        ArrayList<BackupJob> backupJobs;

        public BackupJobListModel(ArrayList<BackupJob> backupJobs) {
            this.backupJobs = backupJobs;
        }

        @Override
        public Object getElementAt(int index) {
            return backupJobs.get(index);
        }

        @Override
        public int getSize() {
            return backupJobs.size();
        }
    }

    private class TaskListModel extends DefaultListModel {

        ArrayList<SlaveTask> slaveTasks;

        public TaskListModel(ArrayList<SlaveTask> slaveTasks) {
            this.slaveTasks = slaveTasks;
        }

        @Override
        public Object getElementAt(int index) {
            return slaveTasks.get(index);
        }

        @Override
        public int getSize() {
            return slaveTasks.size();
        }
    }

    private class PrimaryJobListCellRenderer implements ListCellRenderer {

        public PrimaryJobListCellRenderer() {
        }

        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            PrimaryJob primaryJob = (PrimaryJob) value;
            if (primaryJob == null) {
                return new JLabel();
            }
            return new PrimaryJobSummaryP(primaryJob);
        }
    }

    private class BackupJobListCellRenderer implements ListCellRenderer {

        public BackupJobListCellRenderer() {
        }

        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            BackupJob backupJob = (BackupJob) value;
            if (backupJob == null) {
                return new JLabel();
            }
            return new BackupJobSummaryP(backupJob);
        }
    }

    private class TaskListCellRenderer implements ListCellRenderer {

        public TaskListCellRenderer() {

        }

        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            SlaveTask slaveTask = (SlaveTask) value;

            if (slaveTask == null) {
                LOG.log(Level.FINER, "Task is null in gui");
                return new JLabel();
            }

            return new TaskSummaryP(slaveTask);
        }
    }

    private class JobListMouseListener extends MouseAdapter {

        @Override
        public void mouseClicked(MouseEvent e) {
            if (e.getButton() == MouseEvent.BUTTON1 && e.getClickCount() > 1) {
                JList source = (JList) e.getSource();
                Object selected = source.getSelectedValue();

                if (selected instanceof PrimaryJob) {
                    PrimaryJob pj = (PrimaryJob) selected;
                    show(pj);
                } else if (selected instanceof BackupJob) {
                    BackupJob bj = (BackupJob) selected;
                    show(bj);
                }
            }
        }

        private void show(PrimaryJob pj) {
            PrimaryJobViewer.show(pj, nodeModule.getNodeFSM(), guiFrame);
        }

        private void show(BackupJob bj) {
            BackupJobViewer.show(bj, nodeModule.getNodeFSM(), guiFrame);
        }
    }

    private class TaskListMouseListener extends MouseAdapter {

        @Override
        public void mouseClicked(MouseEvent e) {
            if (e.getButton() == MouseEvent.BUTTON1 && e.getClickCount() > 1) {
                JList source = (JList) e.getSource();
                SlaveTask selected = (SlaveTask) source.getSelectedValue();
                show(selected);
            }
        }

        private void show(SlaveTask selected) {
            TaskViewer.show(selected, nodeModule.getNodeFSM(), guiFrame);
        }
    }
}
