/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.gui;

import java.awt.Color;
import java.awt.Component;
import java.io.IOException;
import java.util.EnumSet;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.ListCellRenderer;
import net.jxta.discovery.DiscoveryService;
import net.jxta.document.Advertisement;
import net.jxta.peer.PeerID;
import p2pMapReduce.networkModule.ConnectionIF;
import p2pMapReduce.networkModule.NetworkModuleIF;
import p2pMapReduce.networkModule.NodeInfo;
import p2pMapReduce.impl.networkModule.jxta.JxtaNetworkModule;
import p2pMapReduce.impl.networkModule.jxta.JxtaNodeInfo;
import p2pMapReduce.impl.networkModule.jxta.JxtaUtil;
import p2pMapReduce.impl.networkModule.jxta.NodeAdvertisement;
import p2pMapReduce.nodeModule.Constants;
import p2pMapReduce.entity.Task.TaskStatus;
import p2pMapReduce.nodeModule.event.EventIF;
import p2pMapReduce.nodeModule.event.EventListener;
import p2pMapReduce.nodeModule.event.EventType;
import p2pMapReduce.nodeModule.event.MessageEvent;
import p2pMapReduce.impl.nodeModule.NodeModuleImpl;
import p2pMapReduce.nodeModule.nodeFsm.NodeFSM;
import p2pMapReduce.nodeModule.nodeFsm.NodeState.StateName;
import p2pMapReduce.test.networkModule.AdvertisementViewer;
import p2pMapReduce.test.networkModule.ListViewer;
import p2pMapReduce.test.networkModule.NetworkModulePrototipe;

/**
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class GUI extends javax.swing.JFrame implements EventListener {

    private NodeInfoListModel slaveListModel;
    private NodeInfoListModel masterListModel;
    private ConnectionListModel connectionListModel;
    private JxtaNetworkModule networkModule;
    private NodeModuleImpl nodeModule;
    private Set<NodeInfo> slavesNodeInfos;
    private Set<NodeInfo> mastersNodeInfos;
    private Set<ConnectionIF> activeConnections;
    AdvertisementViewer v = new AdvertisementViewer("Remote Discovered ADV");

    public static void main(String[] args) {

        new GUI(new NetworkModulePrototipe(null)).setVisible(true);
    }

    /**
     * Creates new form GUI
     * 
     * @deprecated 
     */
    public GUI(NetworkModuleIF networkModule) {
        this.networkModule = (JxtaNetworkModule) networkModule;
        this.slavesNodeInfos = new HashSet<NodeInfo>();
        this.mastersNodeInfos = new HashSet<NodeInfo>();
        this.slaveListModel = new NodeInfoListModel(slavesNodeInfos);
        this.masterListModel = new NodeInfoListModel(mastersNodeInfos);
        this.connectionListModel = new ConnectionListModel();
        this.activeConnections = new HashSet<ConnectionIF>();
//
//        this.networkModule.addEventListener(this);

        initComponents();

//        new SlaveListModelUpdater().start();
//        new MasterListModelUpdater().start();
//        new HeaderPanelUpdater().start();
    }

    public GUI(NodeModuleImpl nodeModule) {
        this.nodeModule = nodeModule;
        this.networkModule = (JxtaNetworkModule) nodeModule.getNetworkModule();
        this.slavesNodeInfos = new HashSet<NodeInfo>();
        this.mastersNodeInfos = new HashSet<NodeInfo>();
        this.slaveListModel = new NodeInfoListModel(slavesNodeInfos);
        this.masterListModel = new NodeInfoListModel(mastersNodeInfos);
        this.connectionListModel = new ConnectionListModel();
        this.activeConnections = new HashSet<ConnectionIF>();

        this.networkModule.addEventListener(this);
        initComponents();

        new SlaveListModelUpdater().start();
        new MasterListModelUpdater().start();
        new HeaderPanelUpdater().start();
    }

    public void handleEvent(final EventIF event) {
//        System.out.println("GUI:NEW EVENT RECEIVED " + event.getType());
        //Per evitare problemi di deadlock con GUI
        new Thread() {

            @Override
            public void run() {
                handle(event);
            }
        }.start();

    }

    private void handle(EventIF event) {
        if (event.getType() == EventType.NEW_CONNECTION) {
            ConnectionIF conn = (ConnectionIF) event.getParam(0);
            synchronized (activeConnections) {
                activeConnections.add(conn);

            }
            update();
        } else if (event.getType() == EventType.CRASH_EVENT) {
            ConnectionIF conn = (ConnectionIF) event.getParam(0);
            synchronized (activeConnections) {
                activeConnections.remove(conn);

            }
            update();
        } else if (event.getType() == EventType.CLOSE_EVENT) {
            ConnectionIF conn = (ConnectionIF) event.getParam(0);
            synchronized (activeConnections) {
                activeConnections.remove(conn);

            }
            update();
        }
    }

    private void update() {
        slaveListModel.update();
        masterListModel.update();
        connectionListModel.update();
        validate();
        repaint();
    }

    private ComboBoxModel getStateComboBoxModel() {
        Set set = EnumSet.allOf(StateName.class);
        return new DefaultComboBoxModel(set.toArray());
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        buttonGroup1 = new javax.swing.ButtonGroup();
        centerPanel = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        masterJList = new javax.swing.JList();
        slaveConnectB = new javax.swing.JButton();
        jScrollPane2 = new javax.swing.JScrollPane();
        activeConnectionJList = new javax.swing.JList();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        disconnectB = new javax.swing.JButton();
        sendB = new javax.swing.JButton();
        jLabel4 = new javax.swing.JLabel();
        jScrollPane3 = new javax.swing.JScrollPane();
        slaveJList = new javax.swing.JList();
        masterConnectB = new javax.swing.JButton();
        headerPanel = new javax.swing.JPanel();
        peerIDLabel = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        isRDVCB = new javax.swing.JCheckBox();
        connToRDVCB = new javax.swing.JCheckBox();
        connToRelayCB = new javax.swing.JCheckBox();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        masterRB = new javax.swing.JRadioButton();
        slaveRB = new javax.swing.JRadioButton();
        jLabel7 = new javax.swing.JLabel();
        coordIDLabel = new javax.swing.JLabel();
        coordinatorCB = new javax.swing.JCheckBox();
        sudPanel = new javax.swing.JPanel();
        showLocalAdvB = new javax.swing.JButton();
        roleCB = new javax.swing.JComboBox();
        changeB = new javax.swing.JButton();
        startRDVB = new javax.swing.JButton();
        stopRDVB = new javax.swing.JButton();
        showRemoteDiscCB = new javax.swing.JCheckBox();
        remoteDiscB = new javax.swing.JButton();
        showRDVB = new javax.swing.JButton();
        stateCB = new javax.swing.JComboBox();
        stateChangeB = new javax.swing.JButton();
        currentStateL = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("GUI");

        masterJList.setFont(new java.awt.Font("Ubuntu", 0, 12));
        masterJList.setModel(masterListModel);
        masterJList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        masterJList.setCellRenderer(new NodeInfoCellRenderer());
        jScrollPane1.setViewportView(masterJList);

        slaveConnectB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        slaveConnectB.setText("Connect");
        slaveConnectB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                slaveConnectBActionPerformed(evt);
            }
        });

        activeConnectionJList.setFont(new java.awt.Font("Ubuntu", 0, 12));
        activeConnectionJList.setModel(connectionListModel);
        activeConnectionJList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jScrollPane2.setViewportView(activeConnectionJList);

        jLabel1.setFont(new java.awt.Font("Ubuntu", 0, 12));
        jLabel1.setText("Active connection list");

        jLabel2.setFont(new java.awt.Font("Ubuntu", 0, 12));
        jLabel2.setText("Slave nodes");

        disconnectB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        disconnectB.setText("Close");
        disconnectB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                disconnectBActionPerformed(evt);
            }
        });

        sendB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        sendB.setText("Send");
        sendB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sendBActionPerformed(evt);
            }
        });

        jLabel4.setFont(new java.awt.Font("Ubuntu", 0, 12));
        jLabel4.setText("Master nodes");

        slaveJList.setFont(new java.awt.Font("Ubuntu", 0, 12));
        slaveJList.setModel(slaveListModel);
        slaveJList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        slaveJList.setCellRenderer(new NodeInfoCellRenderer());
        jScrollPane3.setViewportView(slaveJList);

        masterConnectB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        masterConnectB.setText("Connect");
        masterConnectB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                masterConnectBActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout centerPanelLayout = new javax.swing.GroupLayout(centerPanel);
        centerPanel.setLayout(centerPanelLayout);
        centerPanelLayout.setHorizontalGroup(
            centerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(centerPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(centerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel2)
                    .addComponent(jLabel4)
                    .addComponent(jLabel1)
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 653, Short.MAX_VALUE)
                    .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 653, Short.MAX_VALUE)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 653, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(centerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(sendB, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(disconnectB, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(masterConnectB, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(slaveConnectB, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        centerPanelLayout.setVerticalGroup(
            centerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(centerPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(centerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(centerPanelLayout.createSequentialGroup()
                        .addComponent(jScrollPane3, javax.swing.GroupLayout.PREFERRED_SIZE, 78, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel4))
                    .addComponent(slaveConnectB))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(centerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(centerPanelLayout.createSequentialGroup()
                        .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 78, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel1))
                    .addComponent(masterConnectB))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(centerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(centerPanelLayout.createSequentialGroup()
                        .addComponent(disconnectB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(sendB))
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.PREFERRED_SIZE, 102, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(22, Short.MAX_VALUE))
        );

        centerPanelLayout.linkSize(javax.swing.SwingConstants.VERTICAL, new java.awt.Component[] {disconnectB, masterConnectB, sendB, slaveConnectB});

        peerIDLabel.setFont(new java.awt.Font("Ubuntu", 0, 12));
        peerIDLabel.setText(networkModule.getNodeInfo().getID());

        jLabel3.setFont(new java.awt.Font("Ubuntu", 0, 12));
        jLabel3.setText("PeerID: ");

        isRDVCB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        isRDVCB.setText("isRDV");
        isRDVCB.setEnabled(false);

        connToRDVCB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        connToRDVCB.setText("connToRDV");
        connToRDVCB.setEnabled(false);

        connToRelayCB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        connToRelayCB.setText("connToRelay");
        connToRelayCB.setEnabled(false);

        jLabel5.setFont(new java.awt.Font("Ubuntu", 0, 12));
        jLabel5.setText("GroupID: ");

        jLabel6.setFont(new java.awt.Font("Ubuntu", 0, 12));
        jLabel6.setText(networkModule.getP2pMapReduceGroup().getPeerGroupID().toString());

        masterRB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        masterRB.setText("Master");
        masterRB.setEnabled(false);

        slaveRB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        slaveRB.setText("Slave");
        slaveRB.setEnabled(false);

        jLabel7.setFont(new java.awt.Font("Ubuntu", 0, 12));
        jLabel7.setText("CoordID");

        coordIDLabel.setFont(new java.awt.Font("Ubuntu", 0, 12));
        coordIDLabel.setText(networkModule.getNodeInfo().getID());

        coordinatorCB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        coordinatorCB.setText("Coordinator");
        coordinatorCB.setEnabled(false);

        javax.swing.GroupLayout headerPanelLayout = new javax.swing.GroupLayout(headerPanel);
        headerPanel.setLayout(headerPanelLayout);
        headerPanelLayout.setHorizontalGroup(
            headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(headerPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(headerPanelLayout.createSequentialGroup()
                        .addGroup(headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel5)
                            .addComponent(jLabel3)
                            .addComponent(jLabel7))
                        .addGap(18, 18, 18)
                        .addGroup(headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(peerIDLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 650, Short.MAX_VALUE)
                            .addComponent(coordIDLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 650, Short.MAX_VALUE)
                            .addComponent(jLabel6, javax.swing.GroupLayout.DEFAULT_SIZE, 650, Short.MAX_VALUE)))
                    .addGroup(headerPanelLayout.createSequentialGroup()
                        .addComponent(isRDVCB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(connToRDVCB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(connToRelayCB))
                    .addGroup(headerPanelLayout.createSequentialGroup()
                        .addComponent(masterRB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(coordinatorCB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(slaveRB)))
                .addContainerGap())
        );
        headerPanelLayout.setVerticalGroup(
            headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(headerPanelLayout.createSequentialGroup()
                .addGroup(headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel5)
                    .addComponent(jLabel6))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(peerIDLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 18, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel7)
                    .addComponent(coordIDLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 18, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(isRDVCB)
                    .addComponent(connToRDVCB)
                    .addComponent(connToRelayCB))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(masterRB)
                    .addComponent(coordinatorCB)
                    .addComponent(slaveRB)))
        );

        showLocalAdvB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        showLocalAdvB.setText("Show local adv");
        showLocalAdvB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showLocalAdvBActionPerformed(evt);
            }
        });

        roleCB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        roleCB.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "SLAVE", "MASTER" }));

        changeB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        changeB.setText("Pub");
        changeB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                changeBActionPerformed(evt);
            }
        });

        startRDVB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        startRDVB.setText("Start RDV");
        startRDVB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                startRDVBActionPerformed(evt);
            }
        });

        stopRDVB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        stopRDVB.setText("Stop RDV");
        stopRDVB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stopRDVBActionPerformed(evt);
            }
        });

        showRemoteDiscCB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        showRemoteDiscCB.setText("showRemoteDisc");
        showRemoteDiscCB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showRemoteDiscCBActionPerformed(evt);
            }
        });

        remoteDiscB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        remoteDiscB.setText("RemoteDisc");
        remoteDiscB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                remoteDiscBActionPerformed(evt);
            }
        });

        showRDVB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        showRDVB.setText("ShowRDV");
        showRDVB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showRDVBActionPerformed(evt);
            }
        });

        stateCB.setFont(new java.awt.Font("Ubuntu", 0, 10));
        stateCB.setModel(getStateComboBoxModel());

        stateChangeB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        stateChangeB.setText("Force");
        stateChangeB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stateChangeBActionPerformed(evt);
            }
        });

        currentStateL.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);

        javax.swing.GroupLayout sudPanelLayout = new javax.swing.GroupLayout(sudPanel);
        sudPanel.setLayout(sudPanelLayout);
        sudPanelLayout.setHorizontalGroup(
            sudPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(sudPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(sudPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(currentStateL, javax.swing.GroupLayout.PREFERRED_SIZE, 161, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(sudPanelLayout.createSequentialGroup()
                        .addComponent(stateCB, javax.swing.GroupLayout.PREFERRED_SIZE, 113, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(stateChangeB)))
                .addGap(12, 12, 12)
                .addGroup(sudPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(sudPanelLayout.createSequentialGroup()
                        .addComponent(showLocalAdvB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(roleCB, javax.swing.GroupLayout.PREFERRED_SIZE, 83, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(changeB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(startRDVB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(stopRDVB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(remoteDiscB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(showRDVB))
                    .addComponent(showRemoteDiscCB))
                .addContainerGap(33, Short.MAX_VALUE))
        );
        sudPanelLayout.setVerticalGroup(
            sudPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(sudPanelLayout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(sudPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(sudPanelLayout.createSequentialGroup()
                        .addComponent(currentStateL, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(sudPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(sudPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                .addComponent(stateChangeB)
                                .addComponent(showRemoteDiscCB))
                            .addComponent(stateCB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addGroup(sudPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(showLocalAdvB)
                        .addComponent(roleCB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(changeB)
                        .addComponent(startRDVB)
                        .addComponent(stopRDVB)
                        .addComponent(remoteDiscB)
                        .addComponent(showRDVB))))
        );

        sudPanelLayout.linkSize(javax.swing.SwingConstants.VERTICAL, new java.awt.Component[] {changeB, currentStateL, remoteDiscB, roleCB, showLocalAdvB, showRDVB, showRemoteDiscCB, startRDVB, stateCB, stateChangeB, stopRDVB});

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(headerPanel, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(centerPanel, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(sudPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(headerPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(centerPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(sudPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void slaveConnectBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_slaveConnectBActionPerformed
        Object selected = slaveJList.getSelectedValue();
        if (selected != null) {
            NodeInfo n = (NodeInfo) selected;
            try {
                ConnectionIF conn = networkModule.getConnection(n);
                activeConnections.add(conn);
                update();

            } catch (IOException ex) {
                ex.printStackTrace();
                Logger.getLogger(GUI.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }//GEN-LAST:event_slaveConnectBActionPerformed

    private void disconnectBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_disconnectBActionPerformed
        Object selected = activeConnectionJList.getSelectedValue();
        if (selected != null) {
            ConnectionIF conn = (ConnectionIF) selected;
            try {
                conn.close();
            } catch (IOException ex) {
                ex.printStackTrace();
                Logger.getLogger(GUI.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }//GEN-LAST:event_disconnectBActionPerformed

    private void sendBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sendBActionPerformed
        Object selected = activeConnectionJList.getSelectedValue();
        if (selected != null) {
            ConnectionIF conn = (ConnectionIF) selected;
//            String msg = JOptionPane.showInputDialog("Insert message");
            MessageEvent taskCompletedMsg = new MessageEvent(EventType.TASK_COMPLETED, "String", TaskStatus.ASSIGNED);
            try {
                conn.send(taskCompletedMsg);
            } catch (IOException ex) {
                JOptionPane.showMessageDialog(null, "IOException during send: " + ex.getMessage());
                Logger.getLogger(GUI.class.getName()).log(Level.SEVERE, null, ex);
            }
        }

    }//GEN-LAST:event_sendBActionPerformed

    private void masterConnectBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_masterConnectBActionPerformed
        Object selected = masterJList.getSelectedValue();
        if (selected != null) {
            NodeInfo n = (NodeInfo) selected;
            try {
                ConnectionIF conn = networkModule.getConnection(n);
                activeConnections.add(conn);
                update();

            } catch (IOException ex) {
                ex.printStackTrace();
                Logger.getLogger(GUI.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }//GEN-LAST:event_masterConnectBActionPerformed

    private void showLocalAdvBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showLocalAdvBActionPerformed
        try {
            AdvertisementViewer advV = new AdvertisementViewer("Local ADV");
            DiscoveryService ds = networkModule.getP2pMapReduceGroup().getDiscoveryService();
            Enumeration<Advertisement> list = ds.getLocalAdvertisements(DiscoveryService.ADV, NodeAdvertisement.TYPE_TAG, NodeAdvertisement.getAdvertisementType());
            while (list.hasMoreElements()) {
                Advertisement adv = list.nextElement();
                advV.addADV(adv, ds.getAdvLifeTime(adv), ds.getAdvExpirationTime(adv));
            }

            advV.setVisible(true);
        } catch (IOException ex) {
            ex.printStackTrace();
            Logger.getLogger(GUI.class.getName()).log(Level.SEVERE, null, ex);
        }
    }//GEN-LAST:event_showLocalAdvBActionPerformed

    private void changeBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_changeBActionPerformed
        String sel = roleCB.getSelectedItem().toString();
        if (sel.equalsIgnoreCase("slave")) {
            networkModule.getNodeInfo().setRole(Constants.SLAVE);
        } else {
            networkModule.getNodeInfo().setRole(Constants.MASTER);
        }
        networkModule.publish();
    }//GEN-LAST:event_changeBActionPerformed

    private void startRDVBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_startRDVBActionPerformed
        networkModule.getP2pMapReduceGroup().getRendezVousService().startRendezVous();
    }//GEN-LAST:event_startRDVBActionPerformed

    private void stopRDVBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stopRDVBActionPerformed
        networkModule.getP2pMapReduceGroup().getRendezVousService().stopRendezVous();
    }//GEN-LAST:event_stopRDVBActionPerformed

    private void showRemoteDiscCBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showRemoteDiscCBActionPerformed
        if (showRemoteDiscCB.isSelected()) {
            v.setVisible(true);
        } else {
            v.setVisible(false);
        }
    }//GEN-LAST:event_showRemoteDiscCBActionPerformed

    private void remoteDiscBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_remoteDiscBActionPerformed
        final JButton remoteDiscB_ref = remoteDiscB;
        remoteDiscB_ref.setEnabled(false);
        Runnable runnable = new Runnable() {

            public void run() {
                List<PeerID> rendezvous = networkModule.getP2pMapReduceGroup().getRendezVousService().getLocalRendezVousView();
                String peerID = null;
                if (!rendezvous.isEmpty()) {
                    peerID = rendezvous.get(0).toString();
                }
                DiscoveryService ds = networkModule.getP2pMapReduceGroup().getDiscoveryService();
                Set<Advertisement> ris = JxtaUtil.blockingFindAdv(ds, peerID, ds.ADV, NodeAdvertisement.TYPE_TAG, NodeAdvertisement.getAdvertisementType(), 10 * 1000);
                AdvertisementViewer av = new AdvertisementViewer("Remote Discovery response");
                for (Advertisement adv : ris) {
                    av.addADV(adv, ds.getAdvLifeTime(adv), ds.getAdvExpirationTime(adv));
                }
                av.setVisible(true);
                remoteDiscB_ref.setEnabled(true);
            }
        };

        new Thread(runnable).start();
    }//GEN-LAST:event_remoteDiscBActionPerformed

    private void showRDVBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showRDVBActionPerformed
        List<PeerID> rdvs = networkModule.getP2pMapReduceGroup().getRendezVousService().getLocalRendezVousView();
        ListViewer lv = new ListViewer(rdvs.toArray());
        lv.setVisible(true);
    }//GEN-LAST:event_showRDVBActionPerformed

    private void stateChangeBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stateChangeBActionPerformed
        StateName name = (StateName) stateCB.getSelectedItem();
        nodeModule.forceTransition(name);
    }//GEN-LAST:event_stateChangeBActionPerformed

    private class NodeInfoListModel extends DefaultListModel {

        Set<NodeInfo> nodeInfos;

        public NodeInfoListModel(Set<NodeInfo> nodeInfos) {
            this.nodeInfos = nodeInfos;
        }

        public void update() {
            super.fireContentsChanged(this, 0, nodeInfos.size());
        }

        @Override
        public int getSize() {
            return nodeInfos.size();
        }

        @Override
        public Object getElementAt(int index) {
            int i = 0;
            synchronized (nodeInfos) {
                for (Iterator<NodeInfo> it = nodeInfos.iterator(); it.hasNext();) {
                    NodeInfo nodeInfo = it.next();
                    if (i == index) {
                        return nodeInfo;
                    }
                    i++;
                }
            }
            System.out.println("GUI:richiesto elemento in posizione:" + index + " con dimensione " + nodeInfos.size());
            return null;
        }
    }

    private class NodeInfoCellRenderer extends JLabel implements ListCellRenderer {

      
        public NodeInfoCellRenderer( ) {
           
            setBackground(Color.LIGHT_GRAY);
        }

        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            NodeInfo current = nodeModule.getNodeInfo();
            NodeInfo nodeInfo = (NodeInfo) value;

            if (nodeInfo == null) {
                System.out.println("Null nodeinfo in cell renderer= " + value + " " + index + " slaveNodeInfo.size " + slavesNodeInfos.size() + " masterNodeInfos.size " + mastersNodeInfos.size());
                return this;
            }
            String nodeID = nodeInfo.getID();

            if (nodeID.equals(current.getID())) {
                Color highlighted = nodeInfo.getRole() == current.getRole() ? Color.RED : Color.BLUE;
                setForeground(highlighted);
            } else {
                setForeground(Color.BLACK);
            }
            setText(nodeID + " (" + nodeInfo.getRole() + ")" +" "+((JxtaNodeInfo)nodeInfo).getVersion());

//            Color background;
//            Color foreground;
//
//            // check if this cell represents the current DnD drop location
//            JList.DropLocation dropLocation = list.getDropLocation();
//            if (dropLocation != null
//                    && !dropLocation.isInsert()
//                    && dropLocation.getIndex() == index) {
//
//                background = Color.BLUE;
//                foreground = Color.WHITE;
//
//                // check if this cell is selected
            if (isSelected) {
                setOpaque(true);


                // unselected, and not the DnD drop location
            } else {
                setOpaque(false);
            }

//
//            setBackground(background);
//            setForeground(foreground);

            return this;
        }
    }

    private class ConnectionListModel extends DefaultListModel {

        public ConnectionListModel() {
        }

        public void update() {
            super.fireContentsChanged(this, 0, activeConnections.size());
        }

        public int getSize() {
            return activeConnections.size();
        }

        public Object getElementAt(int index) {
            int i = 0;
            synchronized (activeConnections) {
                for (Iterator<ConnectionIF> it = activeConnections.iterator(); it.hasNext();) {
                    ConnectionIF conn = it.next();
                    if (i == index) {
                        return conn;
                    }
                    i++;
                }
            }
            return null;
        }
    }

    private class ActiveConnectionCellRenderer extends JLabel implements ListCellRenderer {

        public ActiveConnectionCellRenderer() {
            setOpaque(true);
        }

        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {

            ConnectionIF conn = (ConnectionIF) value;
            setText(conn.getRemoteNodeInfo().getID());


//            Color background;
//            Color foreground;
//
//             if (isSelected) {
//                background = Color.RED;
//                foreground = Color.WHITE;
//
//                // unselected, and not the DnD drop location
//            } else {
//                background = Color.WHITE;
//                foreground = Color.BLACK;
//            }
//
//
//            setBackground(background);
//            setForeground(foreground);

            return this;
        }
    }

    private class SlaveListModelUpdater extends Thread {

        @Override
        public void run() {


            long sleepTime = 3 * 1000;

            while (true) {
                try {

                    Set<NodeInfo> slaves = networkModule.getAllSlaves();

                    synchronized (slavesNodeInfos) {
                        slavesNodeInfos.clear();
                        slavesNodeInfos.addAll(slaves);
                    }

                    update();

                    Thread.sleep(sleepTime);
                } catch (InterruptedException ex) {
                    Logger.getLogger(GUI.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
    }

    private class MasterListModelUpdater extends Thread {

        @Override
        public void run() {

            long sleepTime = 3 * 1000;

            while (true) {
                try {

                    Set<NodeInfo> masters = networkModule.getAllMasters();

                    synchronized (mastersNodeInfos) {
                        mastersNodeInfos.clear();
                        mastersNodeInfos.addAll(masters);
                    }

                    update();

                    Thread.sleep(sleepTime);
                } catch (InterruptedException ex) {
                    Logger.getLogger(GUI.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
    }

    private class HeaderPanelUpdater extends Thread {

        long sleepTime = 3 * 1000;

        @Override
        public void run() {
            while (true) {
                try {
                    boolean isRDV = networkModule.getP2pMapReduceGroup().isRendezvous();
                    isRDVCB.setSelected(isRDV);
                    boolean connToRDV = networkModule.getP2pMapReduceGroup().getRendezVousService().isConnectedToRendezVous();
                    connToRDVCB.setSelected(connToRDV);
                    boolean connToRelay = networkModule.getP2pMapReduceGroup().getEndpointService().isConnectedToRelayPeer();
                    connToRelayCB.setSelected(connToRelay);
                    boolean isMaster = networkModule.getNodeInfo().getRole() == Constants.MASTER;
                    masterRB.setSelected(isMaster);
                    slaveRB.setSelected(!isMaster);
                    boolean isCoordinator = networkModule.getNodeModule().isCoordinator();
                    coordinatorCB.setSelected(isCoordinator);
                    coordIDLabel.setText(networkModule.getNodeModule().getCoordinatorID());

                    NodeFSM nodeFSM = nodeModule.getNodeFSM();
                    Class currentState = nodeFSM.getCurrentState().getClass();
                    currentStateL.setText(currentState.getSimpleName());

                    Thread.sleep(sleepTime);
                } catch (InterruptedException ex) {
                    Logger.getLogger(GUI.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JList activeConnectionJList;
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.JPanel centerPanel;
    private javax.swing.JButton changeB;
    private javax.swing.JCheckBox connToRDVCB;
    private javax.swing.JCheckBox connToRelayCB;
    private javax.swing.JLabel coordIDLabel;
    private javax.swing.JCheckBox coordinatorCB;
    private javax.swing.JLabel currentStateL;
    private javax.swing.JButton disconnectB;
    private javax.swing.JPanel headerPanel;
    private javax.swing.JCheckBox isRDVCB;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JButton masterConnectB;
    private javax.swing.JList masterJList;
    private javax.swing.JRadioButton masterRB;
    private javax.swing.JLabel peerIDLabel;
    private javax.swing.JButton remoteDiscB;
    private javax.swing.JComboBox roleCB;
    private javax.swing.JButton sendB;
    private javax.swing.JButton showLocalAdvB;
    private javax.swing.JButton showRDVB;
    private javax.swing.JCheckBox showRemoteDiscCB;
    private javax.swing.JButton slaveConnectB;
    private javax.swing.JList slaveJList;
    private javax.swing.JRadioButton slaveRB;
    private javax.swing.JButton startRDVB;
    private javax.swing.JComboBox stateCB;
    private javax.swing.JButton stateChangeB;
    private javax.swing.JButton stopRDVB;
    private javax.swing.JPanel sudPanel;
    // End of variables declaration//GEN-END:variables
}
