/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.gui;

import p2pMapReduce.Version;
import p2pMapReduce.impl.nodeModule.NodeModuleImpl;

/**
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class GUIFrame extends javax.swing.JFrame {

    private static final String TITLE_PREFIX = "P2P-MapReduce "+Version.VERSION_NUMBER+" - ";

    LogPanel logPanel;
    NetPanel nodePanel;
    ExecutionPanel executionPanel;
    Updater updater;

    public GUIFrame(LogPanel logPanel) {
        this.logPanel = logPanel;
        super.setTitle(TITLE_PREFIX);
        setIconImage(Resource.getInstance().getImageIcon(GUI_Constants.LOGO_IMAGE).getImage());
        setResizable(false);
        initComponents();
        setSize(830, 641);
//        JScrollPane jsp = new JScrollPane(new LogViewerPanel(jtos));
        jTabbedPane1.addTab("Log", logPanel);
        setVisible(true);

        updater = new Updater();
        updater.start();
    }

    @Override
    public void setTitle(String suffix){
        super.setTitle(TITLE_PREFIX+suffix);
    }

    public Updater getUpdater() {
        return updater;
    }

    public void setNodeModule(NodeModuleImpl nodeModule) {
        jTabbedPane1.remove(0);
        nodePanel = new NetPanel(nodeModule, this);
        executionPanel = new ExecutionPanel(nodeModule, this);

        updater.addUpdatable(nodePanel);
        updater.addUpdatable(executionPanel);

        jTabbedPane1.addTab("Net", nodePanel);
        jTabbedPane1.addTab("Exec", executionPanel);
        jTabbedPane1.addTab("Log", logPanel);

        jTabbedPane1.setSelectedComponent(nodePanel);
        pack();
        validate();
        repaint();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jTabbedPane1 = new javax.swing.JTabbedPane();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jTabbedPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 639, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jTabbedPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 446, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTabbedPane jTabbedPane1;
    // End of variables declaration//GEN-END:variables
}
