/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.gui;

import java.io.IOException;
import java.util.Set;
import java.util.concurrent.Callable;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JOptionPane;
import p2pMapReduce.mapreduceModule.Job;
import p2pMapReduce.networkModule.NetworkModuleIF;
import p2pMapReduce.networkModule.NodeInfo;
import p2pMapReduce.nodeModule.event.EventType;
import p2pMapReduce.nodeModule.event.MessageEvent;

/**
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class JobSubmissionFrame extends javax.swing.JFrame {

    NodeInfoListPanel nodeInfoListPanel;
    NetworkModuleIF netModule;
    Job job;

    /** Creates new form SubmissionFrame */
    public JobSubmissionFrame(NetworkModuleIF nodeModule, Job job) {
        this.netModule = nodeModule;
        this.job = job;
        nodeInfoListPanel = new NodeInfoListPanel(nodeModule, getCallable(), "Master nodes: ");
        setTitle("Job " + job.getJobID() + " submission");
        initComponents();
    }

    private Callable<Set<NodeInfo>> getCallable() {
        return new Callable<Set<NodeInfo>>() {

            public Set<NodeInfo> call() throws Exception {
                return netModule.getAllMasters();
            }
        };
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = nodeInfoListPanel;
        submitB = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        submitB.setText("Submit");
        submitB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                submitBActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, 736, Short.MAX_VALUE)
                .addGap(18, 18, 18)
                .addComponent(submitB)
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
                    .addComponent(submitB))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void submitBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_submitBActionPerformed
        NodeInfo selectedMaster = nodeInfoListPanel.getSelectedNode();
        if (selectedMaster != null) {
            System.out.println("Selected master is :" + selectedMaster.getID());
            MessageEvent submitJobMsg = new MessageEvent(EventType.JOB_ASSIGNED, job, netModule.getNodeInfo());
            try {
                netModule.send(selectedMaster, submitJobMsg);
                System.out.println("Job submitted");
                dispose();
            } catch (IOException ex) {
                ex.printStackTrace();
                Logger.getLogger(JobSubmissionFrame.class.getName()).log(Level.SEVERE, null, ex);
            }

        } else {
            JOptionPane.showMessageDialog(this, "Select a node!");
        }
    }//GEN-LAST:event_submitBActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel jPanel1;
    private javax.swing.JButton submitB;
    // End of variables declaration//GEN-END:variables
}
