/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.gui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultHighlighter.DefaultHighlightPainter;
import javax.swing.text.Highlighter;
import p2pMapReduce.util.P2PMapReduceConfig;
import p2pMapReduce.util.LoggingUtil;

/**
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class LogPanel extends JPanel {

    JTextField findTF;
    JButton findB;
    JButton clearB;
    JButton copySelB;
    JButton copyAllB;
    JButton saveSelB;
    JButton saveAllB;
    JCheckBox caseSensitiveCB;
    PrintStream oldStream;
    private JToolBar jToolBar;
    private JTextArea jTextArea;
    private JScrollPane jScrollPane;
    private PrintStream outputStream;
    private Font font = new Font("Courier", Font.PLAIN, 13);
    private String lastSearch = "";
    private int lastIndex;
    private boolean defaultOutput;

    public LogPanel(boolean redirect, boolean defaultOutput) {

        ActionListener finder = new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                find();
            }
        };
        this.defaultOutput = defaultOutput;
        findTF = new JTextField(24);
        findTF.addActionListener(finder);
        findB = new JButton("Find");
        findB.addActionListener(finder);
        caseSensitiveCB = new JCheckBox("Case", false);
        clearB = new JButton("Clear");
        clearB.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                jTextArea.setText("");
            }
        });
        copySelB = new JButton("Copy");
        copySelB.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                String selText = jTextArea.getSelectedText();
                StringSelection data = new StringSelection(selText);
                Toolkit.getDefaultToolkit().getSystemClipboard().setContents(data, data);
            }
        });
        saveSelB = new JButton("Save");
        saveSelB.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                JFileChooser fc = new JFileChooser();
                int ris = fc.showSaveDialog(null);
                if (ris == JFileChooser.APPROVE_OPTION) {
                    File f = fc.getSelectedFile();
                    PrintWriter pw = null;
                    try {
                        pw = new PrintWriter(new BufferedWriter(new FileWriter(f)));
                        pw.print(jTextArea.getSelectedText());

                    } catch (IOException ex) {
                        Logger.getLogger(LogPanel.class.getName()).log(Level.SEVERE, null, ex);
                    } finally {
                        if (pw != null) {
                            pw.close();
                        }
                    }
                }
            }
        });
        copyAllB = new JButton("Copy all");
        copyAllB.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                String text = jTextArea.getText();
                StringSelection data = new StringSelection(text);
                Toolkit.getDefaultToolkit().getSystemClipboard().setContents(data, data);
            }
        });
        saveAllB = new JButton("Save all");
        saveAllB.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                JFileChooser fc = new JFileChooser();
                int ris = fc.showSaveDialog(null);
                if (ris == JFileChooser.APPROVE_OPTION) {
                    File f = fc.getSelectedFile();
                    PrintWriter pw = null;
                    try {
                        pw = new PrintWriter(new BufferedWriter(new FileWriter(f)));
                        pw.print(jTextArea.getText());

                    } catch (IOException ex) {
                        Logger.getLogger(LogPanel.class.getName()).log(Level.SEVERE, null, ex);
                    } finally {
                        if (pw != null) {
                            pw.close();
                        }
                    }
                }
            }
        });
        jTextArea = new JTextArea();
        jToolBar = new JToolBar();
        jScrollPane = new JScrollPane(jTextArea);
        jTextArea.setEditable(false);
        jTextArea.setFont(font);
        jTextArea.addKeyListener(new KeyAdapter() {

            @Override
            public void keyReleased(KeyEvent e) {
                if (e.isControlDown() && e.getKeyCode() == KeyEvent.VK_F) {
                    findTF.requestFocusInWindow();
                }
            }
        });
        if (redirect) {
            redirectSystemStreams();
        }

        initComponents();
    }

    public OutputStream getOutputStream() {
        return outputStream;
    }

    private void updateTextArea(final String text) {
        if (defaultOutput) {
            oldStream.append(text);
        }
        jTextArea.append(text);
        if (jTextArea.getSelectedText() == null) {
            jTextArea.setSelectionStart(jTextArea.getText().length());
            jTextArea.setSelectionEnd(jTextArea.getText().length());
        }

    }

    private void initComponents() {

        FlowLayout fl = new FlowLayout(FlowLayout.LEFT);
        jToolBar.setLayout(fl);
        jToolBar.add(findTF);
        jToolBar.add(caseSensitiveCB);
        jToolBar.add(findB);
        jToolBar.add(clearB);
        jToolBar.add(copySelB);
        jToolBar.add(saveSelB);
        jToolBar.add(copyAllB);
        jToolBar.add(saveAllB);
        final JComboBox logCB = new JComboBox(new DefaultComboBoxModel(getLogLevelsArr()));
        logCB.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                Level sel = (Level) logCB.getSelectedItem();
                Properties p = P2PMapReduceConfig.getConfiguration();
                p.setProperty("logging.level", sel.toString());
                LoggingUtil.setLogging();
            }
        });
        Level lev = Level.parse(P2PMapReduceConfig.getConfiguration().getProperty("logging.level"));
        logCB.setSelectedItem(lev);
        jToolBar.add(logCB);
        setLayout(new BorderLayout());
        add(jToolBar, BorderLayout.PAGE_START);
        add(jScrollPane, BorderLayout.CENTER);
    }

    private Object[] getLogLevelsArr() {
        return new Object[]{Level.OFF, Level.SEVERE, Level.WARNING, Level.INFO, Level.CONFIG, Level.FINE, Level.FINER, Level.FINEST, Level.ALL};
    }

    private void redirectSystemStreams() {
        OutputStream out = new OutputStream() {

            @Override
            public void write(int b) throws IOException {
                updateTextArea(String.valueOf((char) b));
            }

            @Override
            public void write(byte[] b, int off, int len) throws IOException {
                updateTextArea(new String(b, off, len));
            }

            @Override
            public void write(byte[] b) throws IOException {
                write(b, 0, b.length);
            }
        };

        oldStream = System.out;
        outputStream = new PrintStream(out, false);
//        outputStream = out;

        System.setOut(outputStream);
        System.setErr(outputStream);
    }

    private void find() {
        String text = findTF.getText();
        if (text.length() == 0) {
            return;
        }

        String doc = jTextArea.getText();
        if (!caseSensitiveCB.isSelected()) {
            text = text.toLowerCase();
            doc = doc.toLowerCase();
        }
        if (!text.equals(lastSearch)) {
            lastIndex = 0;
            lastSearch = "";
        }

        int index = doc.indexOf(text, lastIndex + lastSearch.length());
        if (index == -1) {
            lastIndex = 0;
            index = doc.indexOf(text, lastIndex + lastSearch.length());
        }
        Highlighter h = jTextArea.getHighlighter();
        h.removeAllHighlights();

        if (index != -1) {
            jTextArea.select(index, index + text.length());
            try {
                h.addHighlight(index, index + text.length(), new DefaultHighlightPainter(jTextArea.getSelectionColor()));
            } catch (BadLocationException ex) {
                Logger.getLogger(LogPanel.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
//        
//        jTextArea.setCaretPosition(index);
//        String s = jTextArea.getSelectedText();
        lastSearch = text;
        lastIndex = index;
    }
}
