/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.gui;

import java.awt.Color;
import java.awt.Component;
import java.io.IOException;
import java.util.Collection;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.ListCellRenderer;
import net.jxta.discovery.DiscoveryService;
import net.jxta.document.Advertisement;
import net.jxta.peer.PeerID;
import net.jxta.protocol.PeerAdvertisement;
import p2pMapReduce.networkModule.ConnectionIF;
import p2pMapReduce.impl.networkModule.jxta.JxtaNetworkModule;
import p2pMapReduce.impl.networkModule.jxta.JxtaNodeInfo;
import p2pMapReduce.impl.networkModule.jxta.JxtaUtil;
import p2pMapReduce.nodeModule.Constants;
import p2pMapReduce.networkModule.NodeInfo;
import p2pMapReduce.entity.Task.TaskStatus;
import p2pMapReduce.nodeModule.event.EventIF;
import p2pMapReduce.nodeModule.event.EventListener;
import p2pMapReduce.nodeModule.event.EventType;
import p2pMapReduce.nodeModule.event.MessageEvent;
import p2pMapReduce.impl.nodeModule.NodeModuleImpl;
import p2pMapReduce.nodeModule.nodeFsm.NodeFSM;
import p2pMapReduce.nodeModule.nodeFsm.NodeState.StateName;
import p2pMapReduce.test.networkModule.AdvertisementViewer;
import p2pMapReduce.test.networkModule.ListViewer;
import p2pMapReduce.util.P2PMapReduceConfig;

/**
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class NetPanel extends javax.swing.JPanel implements EventListener, Updatable {

    private NodeInfoListModel slaveListModel;
    private NodeInfoListModel masterListModel;
    private ConnectionListModel connectionListModel;
    private JxtaNetworkModule networkModule;
    private NodeModuleImpl nodeModule;
    private Set<NodeInfo> slavesNodeInfos;
    private Set<NodeInfo> mastersNodeInfos;
    private Set<ConnectionIF> activeConnections;
    GUIFrame guiFrame;
    NodeIDRenderer nodeIDRenderer;

    public NetPanel(NodeModuleImpl nodeModule, GUIFrame guiFrame) {
        this.nodeModule = nodeModule;
        this.guiFrame = guiFrame;
        this.networkModule = (JxtaNetworkModule) nodeModule.getNetworkModule();
        this.slavesNodeInfos = new HashSet<NodeInfo>();
        this.mastersNodeInfos = new HashSet<NodeInfo>();
        this.slaveListModel = new NodeInfoListModel(slavesNodeInfos);
        this.masterListModel = new NodeInfoListModel(mastersNodeInfos);
        this.connectionListModel = new ConnectionListModel();
        this.activeConnections = new HashSet<ConnectionIF>();
     
        nodeIDRenderer = IDRendererFactory.getNodeIDRenderer();
//        this.networkModule.addEventListener(this);

        initComponents();


        NodeListMouseListener nlsl = new NodeListMouseListener();
        slaveJList.addMouseListener(nlsl);
        masterJList.addMouseListener(nlsl);

    }

    public void handleEvent(final EventIF event) {
//        System.out.println("GUI:NEW EVENT RECEIVED " + event.getType());
        //Per evitare problemi di deadlock con GUI
        new Thread() {

            @Override
            public void run() {
                handle(event);
            }
        }.start();

    }

    /**
     *
     * @param event
     * @deprecated not used
     */
    private void handle(EventIF event) {
        if (event.getType() == EventType.NEW_CONNECTION) {
            ConnectionIF conn = (ConnectionIF) event.getParam(0);
            synchronized (activeConnections) {
                activeConnections.add(conn);

            }
            update();
        } else if (event.getType() == EventType.CRASH_EVENT) {
            ConnectionIF conn = (ConnectionIF) event.getParam(0);
            synchronized (activeConnections) {
                activeConnections.remove(conn);

            }
            update();
        } else if (event.getType() == EventType.CLOSE_EVENT) {
            ConnectionIF conn = (ConnectionIF) event.getParam(0);
            synchronized (activeConnections) {
                activeConnections.remove(conn);

            }
            update();
        }
    }

    public boolean noMoreUpdate() {
        return false;
    }

    public void update() {
        updateLists();
        updateHeaderPanel();

        slaveListModel.update();
        masterListModel.update();
        connectionListModel.update();
        validate();
        repaint();
    }

    private void updateLists() {
        Set<NodeInfo> nodes = networkModule.getAllAvailableNodesFromLocalCache();
        synchronized (slavesNodeInfos) {
            slavesNodeInfos.clear();
            for (Iterator<NodeInfo> it = nodes.iterator(); it.hasNext();) {
                NodeInfo nodeInfo = it.next();
                if (nodeInfo.getRole() == Constants.SLAVE) {
                    slavesNodeInfos.add(nodeInfo);
                    it.remove();
                }
            }
        }

        synchronized (mastersNodeInfos) {
            mastersNodeInfos.clear();
            for (NodeInfo nodeInfo : nodes) {
                if (nodeInfo.getRole() == Constants.MASTER) {
                    mastersNodeInfos.add(nodeInfo);
                } else {
                    System.out.println("Come fa a non essere MASTER?? " + nodeInfo);
                    System.exit(1);
                }
            }
        }


        synchronized (nodeModule) {
            activeConnections.clear();
            activeConnections.addAll(networkModule.getActiveConnection());
        }
    }

    private void updateHeaderPanel() {
        String title = "";
        boolean isRDV = networkModule.getP2pMapReduceGroup().isRendezvous();
        if (isRDV) {
            isRDVCB.setSelected(true);
//            title += "R ";
        } else {
            isRDVCB.setSelected(false);
        }

        boolean connToRDV = networkModule.getP2pMapReduceGroup().getRendezVousService().isConnectedToRendezVous();
        connToRDVCB.setSelected(connToRDV);
        boolean connToRelay = networkModule.getP2pMapReduceGroup().getEndpointService().isConnectedToRelayPeer();
        connToRelayCB.setSelected(connToRelay);
        boolean isMaster = networkModule.getNodeInfo().getRole() == Constants.MASTER;
        if (isMaster) {
            guiFrame.setTitle(title + "M " + nodeIDRenderer.render(nodeModule.getNodeID()));
        } else {
            guiFrame.setTitle(title + "S " + nodeIDRenderer.render(nodeModule.getNodeID()));
        }
        masterRB.setSelected(isMaster);
        slaveRB.setSelected(!isMaster);

        boolean isCoordinator = nodeModule.isCoordinator();
        coordinatorCB.setSelected(isCoordinator);
        String coordinatorID = nodeModule.getCoordinatorID();
        if (coordinatorID != null) {
            coordIDLabel.setText(nodeIDRenderer.render(coordinatorID));
        } else {
            coordIDLabel.setText("");
        }
        NodeFSM nodeFSM = nodeModule.getNodeFSM();
        Class currentState = nodeFSM.getCurrentState().getClass();
        currentStateL.setText(currentState.getSimpleName());
    }

    private ComboBoxModel getStateComboBoxModel() {
        Set set = EnumSet.allOf(StateName.class);
        return new DefaultComboBoxModel(set.toArray());
    }

    private class NodeInfoListModel extends DefaultListModel {

        Set<NodeInfo> nodeInfos;

        public NodeInfoListModel(Set<NodeInfo> nodeInfos) {
            this.nodeInfos = nodeInfos;
        }

        public void update() {
            super.fireContentsChanged(this, 0, nodeInfos.size());
        }

        @Override
        public int getSize() {
            return nodeInfos.size();
        }

        @Override
        public Object getElementAt(int index) {
            int i = 0;
            synchronized (nodeInfos) {
                for (Iterator<NodeInfo> it = nodeInfos.iterator(); it.hasNext();) {
                    NodeInfo nodeInfo = it.next();
                    if (i == index) {
                        return nodeInfo;
                    }
                    i++;
                }
            }
            System.out.println("GUI:richiesto elemento in posizione:" + index + " con dimensione " + nodeInfos.size());
            return null;
        }
    }

    private class NodeInfoCellRenderer extends JLabel implements ListCellRenderer {

        public NodeInfoCellRenderer() {

            setBackground(Color.LIGHT_GRAY);
        }

        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            NodeInfo current = nodeModule.getNodeInfo();
            NodeInfo nodeInfo = (NodeInfo) value;

            if (nodeInfo == null) {
                System.out.println("Null nodeinfo in cell renderer= " + value + " " + index + " slaveNodeInfo.size " + slavesNodeInfos.size() + " masterNodeInfos.size " + mastersNodeInfos.size());
                return this;
            }
            String nodeID = nodeInfo.getID();

            if (nodeID.equals(current.getID())) {
                Color highlighted = nodeInfo.getRole() == current.getRole() ? Color.RED : Color.BLUE;
                setForeground(highlighted);
            } else {
                setForeground(Color.BLACK);
            }
            setText(nodeIDRenderer.render(nodeID) + " (" + nodeInfo.getLoad() + ") " + ((JxtaNodeInfo) nodeInfo).getVersion());

//                // check if this cell is selected
            if (isSelected) {
                setOpaque(true);


                // unselected, and not the DnD drop location
            } else {
                setOpaque(false);
            }


            return this;
        }
    }

    private class ConnectionListModel extends DefaultListModel {

        public ConnectionListModel() {
        }

        public void update() {
            super.fireContentsChanged(this, 0, activeConnections.size());
        }

        @Override
        public int getSize() {
            return activeConnections.size();
        }

        @Override
        public Object getElementAt(int index) {
            int i = 0;
            synchronized (activeConnections) {
                for (Iterator<ConnectionIF> it = activeConnections.iterator(); it.hasNext();) {
                    ConnectionIF conn = it.next();
                    if (i == index) {
                        return conn;
                    }
                    i++;
                }
            }
            return null;
        }
    }

    private class ActiveConnectionCellRenderer extends JLabel implements ListCellRenderer {

        public ActiveConnectionCellRenderer() {
            setBackground(Color.LIGHT_GRAY);
        }

        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {

            ConnectionIF conn = (ConnectionIF) value;
            setText(nodeIDRenderer.render(conn.getRemoteNodeInfo().getID()));

            if (isSelected) {
                setOpaque(true);

            } else {
                setOpaque(false);
            }

            return this;
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        centerPanel = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        masterJList = new javax.swing.JList();
        slaveConnectB = new javax.swing.JButton();
        jScrollPane2 = new javax.swing.JScrollPane();
        activeConnectionJList = new javax.swing.JList();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        disconnectB = new javax.swing.JButton();
        sendB = new javax.swing.JButton();
        jLabel4 = new javax.swing.JLabel();
        jScrollPane3 = new javax.swing.JScrollPane();
        slaveJList = new javax.swing.JList();
        masterConnectB = new javax.swing.JButton();
        headerPanel = new javax.swing.JPanel();
        peerIDLabel = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        isRDVCB = new javax.swing.JCheckBox();
        connToRDVCB = new javax.swing.JCheckBox();
        connToRelayCB = new javax.swing.JCheckBox();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        masterRB = new javax.swing.JRadioButton();
        slaveRB = new javax.swing.JRadioButton();
        jLabel7 = new javax.swing.JLabel();
        coordIDLabel = new javax.swing.JLabel();
        coordinatorCB = new javax.swing.JCheckBox();
        isRelayCB = new javax.swing.JCheckBox();
        sudPanel = new javax.swing.JPanel();
        showLocalAdvB = new javax.swing.JButton();
        roleCB = new javax.swing.JComboBox();
        changeB = new javax.swing.JButton();
        startRDVB = new javax.swing.JButton();
        stopRDVB = new javax.swing.JButton();
        remoteDiscB = new javax.swing.JButton();
        showRDVB = new javax.swing.JButton();
        stateCB = new javax.swing.JComboBox();
        stateChangeB = new javax.swing.JButton();
        currentStateL = new javax.swing.JLabel();
        showRDVB1 = new javax.swing.JButton();
        showRDVB2 = new javax.swing.JButton();
        showRDVB3 = new javax.swing.JButton();
        showRDVB4 = new javax.swing.JButton();

        masterJList.setFont(new java.awt.Font("Ubuntu", 0, 12));
        masterJList.setModel(masterListModel);
        masterJList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        masterJList.setCellRenderer(new NodeInfoCellRenderer());
        jScrollPane1.setViewportView(masterJList);

        slaveConnectB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        slaveConnectB.setText("Connect");
        slaveConnectB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                slaveConnectBActionPerformed(evt);
            }
        });

        activeConnectionJList.setFont(new java.awt.Font("Ubuntu", 0, 12));
        activeConnectionJList.setModel(connectionListModel);
        activeConnectionJList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        activeConnectionJList.setCellRenderer(new ActiveConnectionCellRenderer());
        jScrollPane2.setViewportView(activeConnectionJList);

        jLabel1.setFont(new java.awt.Font("Ubuntu", 0, 12));
        jLabel1.setText("Active connections");

        jLabel2.setFont(new java.awt.Font("Ubuntu", 0, 12));
        jLabel2.setText("Slave nodes");

        disconnectB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        disconnectB.setText("Close");
        disconnectB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                disconnectBActionPerformed(evt);
            }
        });

        sendB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        sendB.setText("Send");
        sendB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sendBActionPerformed(evt);
            }
        });

        jLabel4.setFont(new java.awt.Font("Ubuntu", 0, 12));
        jLabel4.setText("Master nodes");

        slaveJList.setFont(new java.awt.Font("Ubuntu", 0, 12));
        slaveJList.setModel(slaveListModel);
        slaveJList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        slaveJList.setCellRenderer(new NodeInfoCellRenderer());
        jScrollPane3.setViewportView(slaveJList);

        masterConnectB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        masterConnectB.setText("Connect");
        masterConnectB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                masterConnectBActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout centerPanelLayout = new javax.swing.GroupLayout(centerPanel);
        centerPanel.setLayout(centerPanelLayout);
        centerPanelLayout.setHorizontalGroup(
            centerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(centerPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(centerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel2)
                    .addComponent(jLabel4)
                    .addComponent(jLabel1)
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 667, Short.MAX_VALUE)
                    .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 667, Short.MAX_VALUE)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 667, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(centerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(sendB, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(disconnectB, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(masterConnectB, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(slaveConnectB, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        centerPanelLayout.setVerticalGroup(
            centerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(centerPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(centerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(centerPanelLayout.createSequentialGroup()
                        .addComponent(jScrollPane3, javax.swing.GroupLayout.PREFERRED_SIZE, 78, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel4))
                    .addComponent(slaveConnectB))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(centerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(centerPanelLayout.createSequentialGroup()
                        .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 78, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel1))
                    .addComponent(masterConnectB))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(centerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(centerPanelLayout.createSequentialGroup()
                        .addComponent(disconnectB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(sendB))
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.PREFERRED_SIZE, 102, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(22, Short.MAX_VALUE))
        );

        peerIDLabel.setFont(new java.awt.Font("Ubuntu", 0, 12));
        peerIDLabel.setText(networkModule.getNodeInfo().getID());

        jLabel3.setFont(new java.awt.Font("Ubuntu", 0, 12));
        jLabel3.setText("PeerID: ");

        isRDVCB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        isRDVCB.setText("RDV");
        isRDVCB.setEnabled(false);

        connToRDVCB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        connToRDVCB.setText("connToRDV");
        connToRDVCB.setEnabled(false);

        connToRelayCB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        connToRelayCB.setText("connToRelay");
        connToRelayCB.setEnabled(false);

        jLabel5.setFont(new java.awt.Font("Ubuntu", 0, 12));
        jLabel5.setText("GroupID: ");

        jLabel6.setFont(new java.awt.Font("Ubuntu", 0, 12));
        jLabel6.setText(networkModule.getP2pMapReduceGroup().getPeerGroupID().toString());

        masterRB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        masterRB.setText("Master");
        masterRB.setEnabled(false);

        slaveRB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        slaveRB.setText("Slave");
        slaveRB.setEnabled(false);

        jLabel7.setFont(new java.awt.Font("Ubuntu", 0, 12));
        jLabel7.setText("CoordID:");

        coordIDLabel.setFont(new java.awt.Font("Ubuntu", 0, 12));
        coordIDLabel.setText("");

        coordinatorCB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        coordinatorCB.setText("Coordinator");
        coordinatorCB.setEnabled(false);

        isRelayCB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        isRelayCB.setText("Relay");
        isRelayCB.setEnabled(false);

        javax.swing.GroupLayout headerPanelLayout = new javax.swing.GroupLayout(headerPanel);
        headerPanel.setLayout(headerPanelLayout);
        headerPanelLayout.setHorizontalGroup(
            headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(headerPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(headerPanelLayout.createSequentialGroup()
                        .addGroup(headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel5)
                            .addComponent(jLabel3)
                            .addComponent(jLabel7))
                        .addGap(18, 18, 18)
                        .addGroup(headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(peerIDLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 664, Short.MAX_VALUE)
                            .addComponent(coordIDLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 664, Short.MAX_VALUE)
                            .addComponent(jLabel6, javax.swing.GroupLayout.DEFAULT_SIZE, 664, Short.MAX_VALUE)))
                    .addGroup(headerPanelLayout.createSequentialGroup()
                        .addComponent(isRDVCB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(connToRDVCB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(isRelayCB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(connToRelayCB))
                    .addGroup(headerPanelLayout.createSequentialGroup()
                        .addComponent(masterRB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(coordinatorCB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(slaveRB)))
                .addContainerGap())
        );
        headerPanelLayout.setVerticalGroup(
            headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(headerPanelLayout.createSequentialGroup()
                .addGroup(headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel5)
                    .addComponent(jLabel6))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(peerIDLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 18, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel7)
                    .addComponent(coordIDLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 18, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(isRDVCB)
                    .addComponent(connToRDVCB)
                    .addComponent(isRelayCB)
                    .addComponent(connToRelayCB))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(headerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(masterRB)
                    .addComponent(coordinatorCB)
                    .addComponent(slaveRB)))
        );

        showLocalAdvB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        showLocalAdvB.setText("Show local ADV");
        showLocalAdvB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showLocalAdvBActionPerformed(evt);
            }
        });

        roleCB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        roleCB.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "SLAVE", "MASTER" }));

        changeB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        changeB.setText("Pub");
        changeB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                changeBActionPerformed(evt);
            }
        });

        startRDVB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        startRDVB.setText("Start RDV");
        startRDVB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                startRDVBActionPerformed(evt);
            }
        });

        stopRDVB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        stopRDVB.setText("Stop RDV");
        stopRDVB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stopRDVBActionPerformed(evt);
            }
        });

        remoteDiscB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        remoteDiscB.setText("Refresh nodes");
        remoteDiscB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                remoteDiscBActionPerformed(evt);
            }
        });

        showRDVB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        showRDVB.setText("Show RDV");
        showRDVB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showRDVBActionPerformed(evt);
            }
        });

        stateCB.setFont(new java.awt.Font("Ubuntu", 0, 10));
        stateCB.setModel(getStateComboBoxModel());

        stateChangeB.setFont(new java.awt.Font("Ubuntu", 0, 12));
        stateChangeB.setText("Force");
        stateChangeB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stateChangeBActionPerformed(evt);
            }
        });

        currentStateL.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);

        showRDVB1.setFont(new java.awt.Font("Ubuntu", 0, 12));
        showRDVB1.setText("Show EDG");
        showRDVB1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showRDVB1ActionPerformed(evt);
            }
        });

        showRDVB2.setFont(new java.awt.Font("Ubuntu", 0, 12));
        showRDVB2.setText("Disc peers");
        showRDVB2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showRDVB2ActionPerformed(evt);
            }
        });

        showRDVB3.setFont(new java.awt.Font("Ubuntu", 0, 12));
        showRDVB3.setText("Loc peers");
        showRDVB3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showRDVB3ActionPerformed(evt);
            }
        });

        showRDVB4.setFont(new java.awt.Font("Ubuntu", 0, 12));
        showRDVB4.setText("Show REL");
        showRDVB4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showRDVB4ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout sudPanelLayout = new javax.swing.GroupLayout(sudPanel);
        sudPanel.setLayout(sudPanelLayout);
        sudPanelLayout.setHorizontalGroup(
            sudPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(sudPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(sudPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(currentStateL, javax.swing.GroupLayout.PREFERRED_SIZE, 161, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(sudPanelLayout.createSequentialGroup()
                        .addComponent(stateCB, javax.swing.GroupLayout.PREFERRED_SIZE, 113, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(stateChangeB)))
                .addGroup(sudPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(sudPanelLayout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addComponent(roleCB, javax.swing.GroupLayout.PREFERRED_SIZE, 83, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(changeB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(startRDVB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(stopRDVB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(showRDVB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(showRDVB1))
                    .addGroup(sudPanelLayout.createSequentialGroup()
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(showLocalAdvB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(remoteDiscB, javax.swing.GroupLayout.PREFERRED_SIZE, 122, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(showRDVB4)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(showRDVB3, javax.swing.GroupLayout.PREFERRED_SIZE, 95, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(showRDVB2, javax.swing.GroupLayout.PREFERRED_SIZE, 107, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addGap(62, 62, 62))
        );
        sudPanelLayout.setVerticalGroup(
            sudPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(sudPanelLayout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(sudPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(sudPanelLayout.createSequentialGroup()
                        .addComponent(currentStateL, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(sudPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(sudPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                .addComponent(stateChangeB)
                                .addComponent(showLocalAdvB))
                            .addComponent(stateCB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addGroup(sudPanelLayout.createSequentialGroup()
                        .addGroup(sudPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(roleCB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(changeB)
                            .addComponent(startRDVB)
                            .addComponent(stopRDVB)
                            .addComponent(showRDVB)
                            .addComponent(showRDVB1))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(sudPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(showRDVB2)
                            .addComponent(showRDVB3)
                            .addComponent(showRDVB4)
                            .addComponent(remoteDiscB))))
                .addContainerGap())
        );

        sudPanelLayout.linkSize(javax.swing.SwingConstants.VERTICAL, new java.awt.Component[] {changeB, currentStateL, remoteDiscB, roleCB, showLocalAdvB, showRDVB, showRDVB1, showRDVB2, showRDVB3, showRDVB4, startRDVB, stateCB, stateChangeB, stopRDVB});

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addComponent(sudPanel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 758, Short.MAX_VALUE)
                    .addComponent(centerPanel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(headerPanel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(headerPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(centerPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(sudPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents

    private void slaveConnectBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_slaveConnectBActionPerformed
        Object selected = slaveJList.getSelectedValue();
        if (selected != null) {
            NodeInfo n = (NodeInfo) selected;
            try {
                ConnectionIF conn = networkModule.getConnection(n);
                activeConnections.add(conn);
                update();

            } catch (IOException ex) {
                ex.printStackTrace();
                Logger.getLogger(GUI.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
}//GEN-LAST:event_slaveConnectBActionPerformed

    private void disconnectBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_disconnectBActionPerformed
        Object selected = activeConnectionJList.getSelectedValue();
        if (selected != null) {
            ConnectionIF conn = (ConnectionIF) selected;
            try {

                conn.close();
                activeConnections.remove(conn);
                update();

            } catch (IOException ex) {
                ex.printStackTrace();
                Logger.getLogger(GUI.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
}//GEN-LAST:event_disconnectBActionPerformed

    private void sendBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sendBActionPerformed
        Object selected = activeConnectionJList.getSelectedValue();
        if (selected != null) {
            ConnectionIF conn = (ConnectionIF) selected;
            //            String msg = JOptionPane.showInputDialog("Insert message");
            MessageEvent taskCompletedMsg = new MessageEvent(EventType.TASK_COMPLETED, "String", TaskStatus.ASSIGNED);
            try {
                conn.send(taskCompletedMsg);
            } catch (IOException ex) {
                JOptionPane.showMessageDialog(null, "IOException during send: " + ex.getMessage());
                Logger.getLogger(GUI.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }//GEN-LAST:event_sendBActionPerformed

    private void masterConnectBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_masterConnectBActionPerformed
        Object selected = masterJList.getSelectedValue();
        if (selected != null) {
            NodeInfo n = (NodeInfo) selected;
            try {
                ConnectionIF conn = networkModule.getConnection(n);
                activeConnections.add(conn);
                update();

            } catch (IOException ex) {
                ex.printStackTrace();
                Logger.getLogger(GUI.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
}//GEN-LAST:event_masterConnectBActionPerformed

    private void showLocalAdvBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showLocalAdvBActionPerformed

        Set<NodeInfo> allAvailableNodesFromLocalCache = networkModule.getAllAvailableNodesFromLocalCache();
        AdvertisementViewer advV = new AdvertisementViewer("Local ADV");
        for (NodeInfo nodeInfo : allAvailableNodesFromLocalCache) {
            JxtaNodeInfo jxta = (JxtaNodeInfo) nodeInfo;
            advV.addADV(jxta.getNodeADV());
        }

        advV.setVisible(true);

}//GEN-LAST:event_showLocalAdvBActionPerformed

    private void changeBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_changeBActionPerformed
        String sel = roleCB.getSelectedItem().toString();
        if (sel.equalsIgnoreCase("slave")) {
            networkModule.getNodeInfo().setRole(Constants.SLAVE);
        } else {
            networkModule.getNodeInfo().setRole(Constants.MASTER);
        }
        networkModule.publish();
}//GEN-LAST:event_changeBActionPerformed

    private void startRDVBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_startRDVBActionPerformed
        networkModule.getNetPeerGroup().getRendezVousService().startRendezVous();
        networkModule.getP2pMapReduceGroup().getRendezVousService().startRendezVous();
}//GEN-LAST:event_startRDVBActionPerformed

    private void stopRDVBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stopRDVBActionPerformed
        networkModule.getP2pMapReduceGroup().getRendezVousService().stopRendezVous();
}//GEN-LAST:event_stopRDVBActionPerformed

    private void remoteDiscBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_remoteDiscBActionPerformed
        final JButton remoteDiscB_ref = remoteDiscB;
        remoteDiscB_ref.setEnabled(false);
        Runnable runnable = new Runnable() {

            public void run() {
                DiscoveryService ds = networkModule.getP2pMapReduceGroup().getDiscoveryService();
                Set<NodeInfo> ris = networkModule.getAllNodes();
//                AdvertisementViewer av = new AdvertisementViewer("Remote Discovery response");
//                for (NodeInfo nodeinfo : ris) {
//                    JxtaNodeInfo jxtani = (JxtaNodeInfo) nodeinfo;
//                    av.addADV(jxtani.getNodeADV(), ds.getAdvLifeTime(jxtani.getNodeADV()), ds.getAdvExpirationTime(jxtani.getNodeADV()));
//                }
//                av.setVisible(true);
                remoteDiscB_ref.setEnabled(true);
            }
        };

        new Thread(runnable).start();
}//GEN-LAST:event_remoteDiscBActionPerformed

    private void showRDVBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showRDVBActionPerformed
        List<PeerID> rdvs = networkModule.getP2pMapReduceGroup().getRendezVousService().getLocalRendezVousView();
        ListViewer lv = new ListViewer(rdvs.toArray());
        lv.setVisible(true);
}//GEN-LAST:event_showRDVBActionPerformed

    private void stateChangeBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stateChangeBActionPerformed
        StateName name = (StateName) stateCB.getSelectedItem();
        nodeModule.forceTransition(name);
}//GEN-LAST:event_stateChangeBActionPerformed

    private void showRDVB1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showRDVB1ActionPerformed
        List<PeerID> rdvs = networkModule.getP2pMapReduceGroup().getRendezVousService().getLocalEdgeView();
        ListViewer lv = new ListViewer(rdvs.toArray());
        lv.setVisible(true);
    }//GEN-LAST:event_showRDVB1ActionPerformed

    private void showRDVB2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showRDVB2ActionPerformed
        Set<Advertisement> blockingFindAdv = JxtaUtil.blockingFindAdv(networkModule.getP2pMapReduceGroup().getDiscoveryService(), DiscoveryService.PEER, null, null, 3 * 1000);
        AdvertisementViewer advv = new AdvertisementViewer("Network peers");
        Set<String> added = new HashSet<String>();

        for (Advertisement advertisement : blockingFindAdv) {
            PeerAdvertisement padv = (PeerAdvertisement) advertisement;
            if (!added.contains(padv.getPeerID().toString())) {
                advv.addADV(advertisement);
                added.add(padv.getPeerID().toString());
            }
        }
        advv.setVisible(true);
    }//GEN-LAST:event_showRDVB2ActionPerformed

    private void showRDVB3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showRDVB3ActionPerformed
        LinkedList<Advertisement> localADV = JxtaUtil.getLocalADV(networkModule.getP2pMapReduceGroup().getDiscoveryService(), DiscoveryService.PEER, null, null);
        AdvertisementViewer advv = new AdvertisementViewer("local peers adv");
        Set<String> added = new HashSet<String>();
        for (Advertisement advertisement : localADV) {
            PeerAdvertisement padv = (PeerAdvertisement) advertisement;
            if (!added.contains(padv.getPeerID().toString())) {
                advv.addADV(advertisement);
                added.add(padv.getPeerID().toString());
            }
        }
        advv.setVisible(true);
    }//GEN-LAST:event_showRDVB3ActionPerformed

    private void showRDVB4ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showRDVB4ActionPerformed
        Collection<PeerID> rdvs = networkModule.getP2pMapReduceGroup().getEndpointService().getConnectedRelayPeers();
        ListViewer lv = new ListViewer(rdvs.toArray());
        lv.setVisible(true);
    }//GEN-LAST:event_showRDVB4ActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JList activeConnectionJList;
    private javax.swing.JPanel centerPanel;
    private javax.swing.JButton changeB;
    private javax.swing.JCheckBox connToRDVCB;
    private javax.swing.JCheckBox connToRelayCB;
    private javax.swing.JLabel coordIDLabel;
    private javax.swing.JCheckBox coordinatorCB;
    private javax.swing.JLabel currentStateL;
    private javax.swing.JButton disconnectB;
    private javax.swing.JPanel headerPanel;
    private javax.swing.JCheckBox isRDVCB;
    private javax.swing.JCheckBox isRelayCB;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JButton masterConnectB;
    private javax.swing.JList masterJList;
    private javax.swing.JRadioButton masterRB;
    private javax.swing.JLabel peerIDLabel;
    private javax.swing.JButton remoteDiscB;
    private javax.swing.JComboBox roleCB;
    private javax.swing.JButton sendB;
    private javax.swing.JButton showLocalAdvB;
    private javax.swing.JButton showRDVB;
    private javax.swing.JButton showRDVB1;
    private javax.swing.JButton showRDVB2;
    private javax.swing.JButton showRDVB3;
    private javax.swing.JButton showRDVB4;
    private javax.swing.JButton slaveConnectB;
    private javax.swing.JList slaveJList;
    private javax.swing.JRadioButton slaveRB;
    private javax.swing.JButton startRDVB;
    private javax.swing.JComboBox stateCB;
    private javax.swing.JButton stateChangeB;
    private javax.swing.JButton stopRDVB;
    private javax.swing.JPanel sudPanel;
    // End of variables declaration//GEN-END:variables
}
