/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.gui;

import java.awt.Color;
import java.awt.Component;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListCellRenderer;
import p2pMapReduce.impl.networkModule.jxta.JxtaNodeInfo;
import p2pMapReduce.networkModule.NodeInfo;

/**
 * A CellRenderer for a node info
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class NodeInfoCellRenderer extends JLabel implements ListCellRenderer {

    NodeInfo current;

    public NodeInfoCellRenderer(NodeInfo current) {
        this.current = current;
        setBackground(Color.LIGHT_GRAY);
    }

    public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
        NodeInfo nodeInfo = (NodeInfo) value;
        if (nodeInfo == null) {
            System.out.println("Null nodeinfo in cell renderer= " + value + " " + index);
            return this;
        }
        String nodeID = nodeInfo.getID();
        if (nodeID.equals(current.getID())) {
            Color highlighted = nodeInfo.getRole() == current.getRole() ? Color.RED : Color.BLUE;
            setForeground(highlighted);
        } else {
            setForeground(Color.BLACK);
        }
        setText(nodeID + " (" + nodeInfo.getRole() + ")" + " " + ((JxtaNodeInfo) nodeInfo).getVersion());

        if (isSelected) {
            setOpaque(true);
            // unselected, and not the DnD drop location
        } else {
            setOpaque(false);
        }

        return this;
    }
}
