/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.gui;

import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.Callable;
import java.util.logging.Level;
import java.util.logging.Logger;
import p2pMapReduce.networkModule.NetworkModuleIF;
import p2pMapReduce.networkModule.NodeInfo;

/**
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class NodeInfoListPanel extends javax.swing.JPanel {

    public static final int UPDATE_INTERVAL = 3 * 1000;
    private NodeInfoListModel nodeListModel;
    private NetworkModuleIF networkModule;
    private Callable<Set<NodeInfo>> updater;
    private final Set<NodeInfo> nodeInfos;
    private String labelText;

    public NodeInfoListPanel(NetworkModuleIF netModule, Callable<Set<NodeInfo>> updater, String labelText) {
        this.updater = updater;
        this.labelText = labelText;
        this.networkModule = netModule;
        this.nodeInfos = new HashSet<NodeInfo>();
        this.nodeListModel = new NodeInfoListModel(nodeInfos);

        initComponents();

        new UpdaterThread().start();
    }

    public NodeInfo getSelectedNode() {
        return (NodeInfo) nodeJList.getSelectedValue();
    }

    private void update() {
        nodeListModel.update();
        infoLabel.setText(labelText + " " + nodeInfos.size());
        validate();
        repaint();
    }

    private class UpdaterThread extends Thread {

        @Override
        public void run() {

            while (true) {
                try {

                    Set<NodeInfo> nodes = updater.call();

                    synchronized (nodeInfos) {
                        nodeInfos.clear();
                        nodeInfos.addAll(nodes);

                    }

                    update();

                    Thread.sleep(UPDATE_INTERVAL);

                } catch (Exception ex) {
                    Logger.getLogger(GUI.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane3 = new javax.swing.JScrollPane();
        nodeJList = new javax.swing.JList();
        infoLabel = new javax.swing.JLabel();

        nodeJList.setFont(new java.awt.Font("Ubuntu", 0, 12));
        nodeJList.setModel(nodeListModel);
        nodeJList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        nodeJList.setCellRenderer(new NodeInfoCellRenderer(networkModule.getNodeInfo()));
        jScrollPane3.setViewportView(nodeJList);

        infoLabel.setFont(new java.awt.Font("Ubuntu", 0, 12));
        infoLabel.setText("Nodes");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(infoLabel)
                    .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 632, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(infoLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane3, javax.swing.GroupLayout.PREFERRED_SIZE, 78, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel infoLabel;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JList nodeJList;
    // End of variables declaration//GEN-END:variables
}
