/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.gui;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.LinkedList;
import java.util.logging.Logger;
import javax.swing.JFrame;
import net.jxta.id.IDFactory;
import p2pMapReduce.entity.BackupJob;
import p2pMapReduce.entity.JobTask;
import p2pMapReduce.entity.SlaveTask;
import p2pMapReduce.entity.Task.TaskStatus;
import p2pMapReduce.mapreduceModule.TaskID;
import p2pMapReduce.nodeModule.nodeFsm.NodeFSM;

/**
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class TaskViewer extends javax.swing.JPanel implements Updatable {

    private static final transient Logger LOG = Logger.getLogger(TaskViewer.class.getName());

    public static void show(SlaveTask st, NodeFSM nodeFSM, GUIFrame guiFrame) {
        TaskAttIDRenderer taskAttIDRenderer = IDRendererFactory.getTaskAttIDRenderer();
        JFrame frame = new JFrame("Task " + taskAttIDRenderer.render(st.getTask().getTaskAttID()));
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        TaskViewer taskViewer = new TaskViewer(st, nodeFSM, frame);
        frame.setContentPane(taskViewer);
        frame.pack();
        Locator.locateCenter(frame);
        frame.setVisible(true);
        guiFrame.getUpdater().addUpdatable(taskViewer);
    }
    SlaveTask slaveTask;
    NodeFSM nodeFSM;
    boolean noMoreUpdate;

    public TaskViewer(SlaveTask st, NodeFSM nodeFSM, JFrame frame) {
        this.slaveTask = st;
        this.nodeFSM = nodeFSM;
        initComponents();

        String taskAttID = slaveTask.getTask().getTaskAttIDAsString();
        char type = TaskID.getRepresentingCharacter(slaveTask.getTask().getTaskType());
        int charIndex = taskAttID.lastIndexOf(type);

//        taskIDL.setText(taskAttID.substring(charIndex));
        taskIDL.setText(slaveTask.getTask().getTaskAttID().toString());
        jobIDL.setText(IDRendererFactory.getJobIDRenderer().render(slaveTask.getTask().getTaskID().getJobID()));
        frame.addWindowListener(new WindowAdapter() {

            @Override
            public void windowClosing(WindowEvent e) {
                noMoreUpdate = true;
            }

            @Override
            public void windowClosed(WindowEvent e) {
                noMoreUpdate = true;
            }
        });
    }

    public boolean noMoreUpdate() {
        return noMoreUpdate;
    }

    public void update() {
        TaskStatus taskStatus = slaveTask.getTaskStatus();
        if (taskStatus == TaskStatus.FAILED || taskStatus == TaskStatus.SUCCESS) {
            noMoreUpdate = true;
        }

        primaryID.setText(IDRendererFactory.getNodeIDRenderer().render(slaveTask.getPrimaryID()));
        taskStatusL.setText(taskStatus.toString());
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        javax.swing.JLabel jLabel3 = new javax.swing.JLabel();
        jobIDL = new javax.swing.JLabel();
        javax.swing.JLabel jLabel4 = new javax.swing.JLabel();
        taskStatusL = new javax.swing.JLabel();
        javax.swing.JLabel jLabel5 = new javax.swing.JLabel();
        primaryID = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        taskIDL = new javax.swing.JLabel();

        jLabel3.setText("Job ID");

        jobIDL.setFont(new java.awt.Font("Ubuntu", 0, 14));

        jLabel4.setText("Task status");

        jLabel5.setText("Primary node ID");

        primaryID.setFont(new java.awt.Font("Ubuntu", 0, 14));

        jLabel8.setText("Task ID");

        taskIDL.setFont(new java.awt.Font("Ubuntu", 0, 14));

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jLabel4)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(taskStatusL, javax.swing.GroupLayout.DEFAULT_SIZE, 627, Short.MAX_VALUE))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel3)
                            .addComponent(jLabel5))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(primaryID, javax.swing.GroupLayout.DEFAULT_SIZE, 627, Short.MAX_VALUE)
                            .addComponent(jobIDL, javax.swing.GroupLayout.DEFAULT_SIZE, 627, Short.MAX_VALUE)))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jLabel8)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(taskIDL, javax.swing.GroupLayout.DEFAULT_SIZE, 627, Short.MAX_VALUE)))
                .addContainerGap())
        );

        jPanel1Layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {jLabel3, jLabel4, jLabel5, jLabel8});

        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel8)
                    .addComponent(taskIDL))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(jobIDL, javax.swing.GroupLayout.PREFERRED_SIZE, 18, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel5)
                    .addComponent(primaryID, javax.swing.GroupLayout.PREFERRED_SIZE, 18, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel4)
                    .addComponent(taskStatusL, javax.swing.GroupLayout.PREFERRED_SIZE, 18, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(60, 60, 60))
        );

        jPanel1Layout.linkSize(javax.swing.SwingConstants.VERTICAL, new java.awt.Component[] {jLabel3, jLabel4, jLabel5, jobIDL, primaryID, taskIDL, taskStatusL});

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, 119, javax.swing.GroupLayout.PREFERRED_SIZE)
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel8;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JLabel jobIDL;
    private javax.swing.JLabel primaryID;
    private javax.swing.JLabel taskIDL;
    private javax.swing.JLabel taskStatusL;
    // End of variables declaration//GEN-END:variables
}
