/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.gui;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Updater extends Thread {

    private static final int UPDATE_INTERVAL = 5 * 1000;
    LinkedList<Updatable> updatables = new LinkedList<Updatable>();

    public Updater() {
        setName("Updater");
    }

    public synchronized void addUpdatable(Updatable updatable) {
        updatables.add(updatable);
    }

    public synchronized boolean removeUpdatable(Updatable updatable) {
        return updatables.remove(updatable);
    }

    @Override
    public void run() {
        while (true) {
            synchronized (this) {
                for (Iterator<Updatable> it = updatables.iterator(); it.hasNext();) {
                    Updatable updatable = it.next();
                    if (updatable.noMoreUpdate()) {
                        it.remove();
                    } else {
                        updatable.update();
                    }
                }
            }

            System.gc();

            try {
                Thread.sleep(UPDATE_INTERVAL);
            } catch (InterruptedException ex) {
                Logger.getLogger(ExecutionPanel.class.getName()).log(Level.SEVERE, null, ex);
            }
        }

    }
}
