/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.gui;

import p2pMapReduce.Version;
import p2pMapReduce.entity.JobTask;
import p2pMapReduce.mapreduceModule.Job;
import p2pMapReduce.mapreduceModule.JobID;
import p2pMapReduce.mapreduceModule.JobStatus;
import p2pMapReduce.nodeModule.UserNode;
import p2pMapReduce.nodeModule.UserNode.UserJob;
import p2pMapReduce.nodeModule.event.EventIF;
import p2pMapReduce.nodeModule.event.EventListener;

/**
 * A frame to show information about jobs submitted by the
 * user
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class UserJobFrame extends javax.swing.JFrame implements EventListener, Updatable {

    String jobID;
    String primaryID;
    String titlePrefix;
    Job job;
    UserNode userNode;
    boolean noMoreUpdate;

    /** Creates new form UserJobFrame */
    public UserJobFrame(Job job, UserNode usernode) {
        this.job = job;
        this.userNode = usernode;
        titlePrefix = "P2P-MapReduce v " + Version.VERSION_NUMBER + " - ";
        titlePrefix += " U " + IDRendererFactory.getNodeIDRenderer().render(usernode.getNodeID());
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);

        setTitle(titlePrefix);
        setIconImage(Resource.getInstance().getImageIcon(GUI_Constants.LOGO_IMAGE).getImage());
        initComponents();

    }

    public boolean noMoreUpdate() {
        return noMoreUpdate;
    }

    public void update() {
        if (primaryID == null && jobID != null) {
            UserJob get = userNode.getAssignedJob().get(jobID);
            if (get != null) {
                primaryID = get.getPrimaryID();
                primaryIDL.setText(IDRendererFactory.getNodeIDRenderer().render(primaryID));
                jobStatusL.setText(JobStatus.State.RUNNING.toString());
            }
        }
    }

    public void handleEvent(EventIF event) {


        switch (event.getType()) {
            case JOB_ID_REPLY:
                jobID = event.getParam(1).toString();
                jobIDL.setText(IDRendererFactory.getJobIDRenderer().render(JobID.forName(jobID)));
                break;
            case PRIMARY_UPDATE:
                if (event.getParam(0).toString().equals(jobID)) {
                    primaryID = event.getParam(1).toString();
                    String render = IDRendererFactory.getNodeIDRenderer().render(primaryID);
                    primaryIDL.setText(render);
                    jTextArea1.append("Primary node updated. New primary node is:" + render + "\n");
                }
                break;
            case REDUCE_COMPLETED:
                JobTask jobTask = (JobTask) event.getParam(0);
                if (jobTask.getTask().getJobID().equals(jobID)) {
                    int reduceID = jobTask.getTask().getTaskID().getId();
                    jTextArea1.append("Reducer " + reduceID + " completed\n");
                }
                break;
            case JOB_COMPLETED:
                String complJobID = (String) event.getParam(0);
                if (complJobID.equals(jobID)) {
                    JobStatus.State state = (JobStatus.State) event.getParam(1);
                    jobStatusL.setText(state.toString());
                    jTextArea1.append("Job completed with status " + state);
                }
                noMoreUpdate = true;
                break;
            default:
                ;
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel1 = new javax.swing.JLabel();
        jobIDL = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        primaryIDL = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jobStatusL = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTextArea1 = new javax.swing.JTextArea();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        jLabel1.setText("JobID:");

        jLabel3.setText("Primary ID:");

        jLabel5.setText("Job status:");

        jTextArea1.setColumns(20);
        jTextArea1.setEditable(false);
        jTextArea1.setRows(5);
        jScrollPane1.setViewportView(jTextArea1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 596, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                        .addComponent(jLabel1)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jobIDL, javax.swing.GroupLayout.DEFAULT_SIZE, 513, Short.MAX_VALUE))
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                        .addComponent(jLabel3)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(primaryIDL, javax.swing.GroupLayout.DEFAULT_SIZE, 513, Short.MAX_VALUE))
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                        .addComponent(jLabel5)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jobStatusL, javax.swing.GroupLayout.DEFAULT_SIZE, 513, Short.MAX_VALUE)))
                .addContainerGap())
        );

        layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {jLabel1, jLabel3, jLabel5});

        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(jobIDL, javax.swing.GroupLayout.PREFERRED_SIZE, 18, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(primaryIDL))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel5)
                    .addComponent(jobStatusL))
                .addGap(18, 18, 18)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 240, Short.MAX_VALUE)
                .addContainerGap())
        );

        layout.linkSize(javax.swing.SwingConstants.VERTICAL, new java.awt.Component[] {jobIDL, jobStatusL, primaryIDL});

        pack();
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTextArea jTextArea1;
    private javax.swing.JLabel jobIDL;
    private javax.swing.JLabel jobStatusL;
    private javax.swing.JLabel primaryIDL;
    // End of variables declaration//GEN-END:variables
}
