/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.impl.networkModule.jxta;

import java.util.logging.Level;
import java.util.logging.Logger;
import net.jxta.endpoint.Message;
import net.jxta.endpoint.MessageElement;
import net.jxta.peer.PeerID;
import net.jxta.peergroup.PeerGroup;
import net.jxta.pipe.PipeMsgEvent;
import net.jxta.pipe.PipeMsgListener;
import net.jxta.util.JxtaBiDiPipe;
import net.jxta.util.PipeEventListener;
import net.jxta.util.PipeStateListener;
import p2pMapReduce.nodeModule.NodeModuleIF;

/**
 * Every time a new connection from a remote node is received a new ConnectionHandler
 * is associated to this connection.
 * ConnectionHandler receive message and state events and forward it to the node module
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 * @deprecated not used
 */
public class JxtaBidiPipeConnectionHandler extends Thread implements PipeMsgListener, PipeStateListener, PipeEventListener {

    public static final int RECEIVE_TIMEOUT = 10 * 1000;
    NodeModuleIF nodeModule;
    JxtaBiDiPipe bidiPipe;
    PeerGroup peerGroup;

    public JxtaBidiPipeConnectionHandler(JxtaBiDiPipe bidiPipe, PeerGroup peerGroup) {
        this.bidiPipe = bidiPipe;
        this.peerGroup = peerGroup;
//        this.bidiPipe.setMessageListener(this);
        this.bidiPipe.setPipeStateListener(this);
        this.bidiPipe.setPipeEventListener(this);

    }

    @Override
    public void run() {

        while (true) {
            try {
                System.out.println("Try to receive a message");
//                Message m = bidiPipe.getInputPipe().waitForMessage();
                Message m = bidiPipe.getMessage(RECEIVE_TIMEOUT);
                if (m != null) {
                    boolean alive = check();
                    System.out.println("Received message " + m.getMessageElements().next());
                    System.out.println("alive = " + alive);
                } else {
                    boolean alive = check();
                    System.out.println("RECEIVED NULL MESSAGE");
                    System.out.println("alive = " + alive);
                }

//            } catch (IOException ex) {
//                Logger.getLogger(ConnectionHandler.class.getName()).log(Level.SEVERE, null, ex);
            } catch (InterruptedException ex) {
                Logger.getLogger(JxtaBidiPipeConnectionHandler.class.getName()).log(Level.SEVERE, null, ex);
            }

        }
    }

    public void pipeMsgEvent(PipeMsgEvent event) {
        MessageElement msgElement = event.getMessage().getMessageElement(JxtaBidiPipeConnection.MESSAGE_NAMESPACE_NAME, JxtaBidiPipeConnection.MESSAGE_ELEMENT_NAME);

        Logger.getLogger(JxtaBidiPipeConnection.class.getName()).log(Level.INFO, "Received message " + msgElement.toString());

//        nodeModule.handleMessage(msgElement);

    }

    public void stateEvent(Object source, int event) {
        switch (event) {
            case PipeStateListener.PIPE_OPENED_EVENT:
                Logger.getLogger(JxtaBidiPipeConnectionHandler.class.getName()).log(Level.INFO, "PIPE OPENED");
                break;
            case PipeStateListener.PIPE_CLOSED_EVENT:
                Logger.getLogger(JxtaBidiPipeConnectionHandler.class.getName()).log(Level.INFO, "PIPE CLOSED");
                break;
            case PipeStateListener.PIPE_FAILED_EVENT:
                Logger.getLogger(JxtaBidiPipeConnectionHandler.class.getName()).log(Level.INFO, "PIPE FAILED");
                break;
            default:
                throw new AssertionError();
        }
    }

    public void pipeEvent(int event) {
        switch (event) {
            case PipeStateListener.PIPE_OPENED_EVENT:
                Logger.getLogger(JxtaBidiPipeConnectionHandler.class.getName()).log(Level.INFO, "PIPE OPENED");
                break;
            case PipeStateListener.PIPE_CLOSED_EVENT:
                Logger.getLogger(JxtaBidiPipeConnectionHandler.class.getName()).log(Level.INFO, "PIPE CLOSED");
                break;
            case PipeStateListener.PIPE_FAILED_EVENT:
                Logger.getLogger(JxtaBidiPipeConnectionHandler.class.getName()).log(Level.INFO, "PIPE FAILED");
                break;
            default:
                throw new AssertionError();
        }
    }

    private boolean check() {
        PeerID remoteId = bidiPipe.getRemotePeerAdvertisement().getPeerID();
        return peerGroup.getEndpointService().isReachable(remoteId, false);
    }
}
