/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.impl.networkModule.jxta;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import net.jxta.peergroup.PeerGroup;
import net.jxta.protocol.PipeAdvertisement;
import net.jxta.util.JxtaBiDiPipe;
import net.jxta.util.JxtaServerPipe;
import p2pMapReduce.nodeModule.event.InnerEvent;
import p2pMapReduce.nodeModule.event.EventNotifier;
import p2pMapReduce.nodeModule.event.EventType;

/**
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class JxtaBidiPipeServerThread extends Thread {

    private static final transient Logger LOG = Logger.getLogger(JxtaBidiPipeServerThread.class.getName());
    JxtaNetworkModule netModule;
    PipeAdvertisement pipeADV;
    PeerGroup peerGroup;
    EventNotifier eventNotifier;

    public JxtaBidiPipeServerThread(JxtaNetworkModule netMod, EventNotifier eventNotifier, PeerGroup peerGroup, PipeAdvertisement pipeADV) {
        super();
        this.netModule = netMod;
        this.eventNotifier = eventNotifier;
        this.peerGroup = peerGroup;
        this.pipeADV = pipeADV;
    }

    @Override
    public void run() {
        try {
            JxtaServerPipe serverPipe = new JxtaServerPipe(peerGroup, pipeADV);
            // block forever until a connection is accepted
            serverPipe.setPipeTimeout(0);
            while (true) {
                try {
                    LOG.log(Level.INFO, "Waiting pipe connecion...");
                    JxtaBiDiPipe bipipe = serverPipe.accept();
                    if (bipipe != null) {
                        LOG.log(Level.INFO, "JxtaBidiPipe accepted from " + bipipe.getRemotePeerAdvertisement().getPeerID() + " : " + bipipe.getRemotePipeAdvertisement().getPipeID());
                        JxtaBidiPipeConnection bidiPipeConn = new JxtaBidiPipeConnection(bipipe, peerGroup, netModule.getNodeInfo(), netModule, eventNotifier);
                        netModule.putConnection(bidiPipeConn.getRemoteNodeInfo().getID(), bidiPipeConn);
                        notifyNewConnection(bidiPipeConn);

                    } else {
                        LOG.log(Level.SEVERE, "NULL bidipipe received from accept!!!");
                    }
                } catch (Throwable t) {
                    LOG.log(Level.SEVERE, "Throwable during accept: " + t.getLocalizedMessage(), t);
                }
            }
        } catch (IOException ex) {
            LOG.log(Level.SEVERE, null, ex);
        } finally {
            LOG.log(Level.SEVERE, "SERVER THREAD EXITING!!! NO MORE CONNECTION FROM REMOTE NODE WILL BE ACCEPTED");
            throw new RuntimeException("SERVER THREAD EXITING!!! NO MORE CONNECTION FROM REMOTE NODE WILL BE ACCEPTED");
        }
    }

    private void notifyNewConnection(JxtaBidiPipeConnection bidiPipeConn) {
        InnerEvent event = new InnerEvent(EventType.NEW_CONNECTION, new Object[]{bidiPipeConn});
        eventNotifier.notifyEvent(event);
    }
}
