/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.impl.networkModule.jxta;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.util.logging.Level;
import java.util.logging.Logger;
import net.jxta.codat.CodatID;
import net.jxta.document.AdvertisementFactory;
import net.jxta.id.IDFactory;
import net.jxta.peergroup.PeerGroup;
import net.jxta.pipe.PipeID;
import net.jxta.pipe.PipeService;
import net.jxta.protocol.PipeAdvertisement;
import p2pMapReduce.nodeModule.Constants;
import p2pMapReduce.networkModule.NodeInfo;

/**
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class NodeAdvertisementUtil {

    public static final long INITIAL_VERSION_VALUE = 1L;
    /**
     * Version increment to publish the node adv of a failed node
     */
    public static final long FAILED_VERSION_INCREMENT = 3L;

    public static NodeAdvertisement newNodeAdvertisement(PeerGroup pg) {
        NodeAdvertisement nodeADV = new NodeAdvertisement();
        nodeADV.setID(createNodeAdvertisementID(INITIAL_VERSION_VALUE, pg));
        nodeADV.setVersion(INITIAL_VERSION_VALUE);
        nodeADV.setAvailable(true);
        nodeADV.setRole(Constants.SLAVE);
        nodeADV.setLoad(0L);
        nodeADV.setPeerID(pg.getPeerID().toString());
        nodeADV.setPipeID(createPipeID(pg).toString());
        return nodeADV;
    }

    public static NodeAdvertisement createNodeAdvertisement(long version, PeerGroup peerGroup, NodeAdvertisement nodeADV) {
        NodeAdvertisement ris = new NodeAdvertisement();
        ris.setID(createNodeAdvertisementID(version, peerGroup, nodeADV.getPeerID()));
        ris.setVersion(version);
        ris.setAvailable(nodeADV.isAvailable());
        ris.setRole(nodeADV.getRole());
        ris.setLoad(nodeADV.getLoad());
        ris.setPeerID(nodeADV.getPeerID().toString());
        ris.setPipeID(nodeADV.getPipeID().toString());
        return ris;
    }

    @Deprecated
    public static NodeAdvertisement createNodeAdvertisement(PeerGroup pg) {
        NodeAdvertisement nodeADV = new NodeAdvertisement();
        nodeADV.setID(createNodeAdvertisementID(pg));
        nodeADV.setRole(Constants.SLAVE);
        nodeADV.setLoad(0);
        nodeADV.setPeerID(pg.getPeerID().toString());
        nodeADV.setPipeID(createPipeID(pg).toString());
        return nodeADV;
    }

    public static NodeAdvertisement createNodeAdvertisement(NodeInfo nodeInfo) {
        //TODO per ora cast ma se necessario includere peerid nel nodeAdv
        return ((JxtaNodeInfo) nodeInfo).getNodeADV();
//        NodeAdvertisement nodeADV = new NodeAdvertisement();
//        nodeADV.setID(createNodeID());
//        nodeADV.setRole(nodeInfo.getRole());
//        nodeADV.setLoad(nodeInfo.getLoad());
//        nodeADV.setPipeID(create);
//        return nodeADV;
    }

    /**
     * Create a PipeAdertisement for this node.
     * The PipeID is created from group and peer id.
     * @return
     */
    public static PipeAdvertisement createPipeAdvertisement(PeerGroup pg) {
        PipeAdvertisement pipeADV = (PipeAdvertisement) AdvertisementFactory.newAdvertisement(PipeAdvertisement.getAdvertisementType());
        pipeADV.setPipeID(createPipeID(pg));
        pipeADV.setType(PipeService.UnicastType);
        pipeADV.setName("P2PMapReduce pipe");
        return pipeADV;
    }

    public static PipeAdvertisement createPipeAdvertisement(NodeInfo connectionEndPoint) {
        return createPipeAdvertisement(createNodeAdvertisement(connectionEndPoint));
    }

    public static PipeAdvertisement createPipeAdvertisement(NodeAdvertisement nodeADV) {
        PipeAdvertisement pipeADV = (PipeAdvertisement) AdvertisementFactory.newAdvertisement(PipeAdvertisement.getAdvertisementType());
        pipeADV.setPipeID(PipeID.create(URI.create(nodeADV.getPipeID())));
        pipeADV.setType(PipeService.UnicastType);
        pipeADV.setName("P2PMapReduce pipe of " + nodeADV.getPeerID());
        return pipeADV;
    }

    /**
     *
     * @return
     * @deprecated non considera numero di versione nell'id
     */
    @Deprecated
    public static CodatID createNodeAdvertisementID(PeerGroup pg) {
        try {
            //Codice preso da PeerAdvertisement per creare l'ID del PeerAdvertisement stesso
            //TODO if necessary use the peer id instead
            String hashValue = NodeAdvertisement.getAdvertisementType() + pg.getPeerGroupID().getUniqueValue().toString() + pg.getPeerID().getUniqueValue().toString();
            byte[] seed = hashValue.getBytes("UTF-8");
            return IDFactory.newCodatID(pg.getPeerGroupID(), seed, new ByteArrayInputStream(seed));
            //        PeerID peerid = p2pMapReduceGroup.getPeerID();
            //        CodatID codat = IDFactory.newCodatID(PeerGroupID.worldPeerGroupID, peerid.toString().getBytes());
            //        return codat;
        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(JxtaNetworkModule.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(JxtaNetworkModule.class.getName()).log(Level.SEVERE, null, ex);
        }

        return null;
    }

    /**
     * Generate a new NodeADV ID for this peer
     *
     * @param version
     * @param pg
     * @return
     */
    public static CodatID createNodeAdvertisementID(Long version, PeerGroup pg) {
        try {
            //Codice preso da PeerAdvertisement per creare l'ID del PeerAdvertisement stesso
            String hashValue = version.toString() + NodeAdvertisement.getAdvertisementType() + pg.getPeerGroupID().getUniqueValue().toString() + pg.getPeerID().getUniqueValue().toString();
            byte[] seed = hashValue.getBytes("UTF-8");
//            byte[] variant = version.toString().getBytes("UTF-8");

            return IDFactory.newCodatID(pg.getPeerGroupID(), seed, new ByteArrayInputStream(seed));
            //        PeerID peerid = p2pMapReduceGroup.getPeerID();
            //        CodatID codat = IDFactory.newCodatID(PeerGroupID.worldPeerGroupID, peerid.toString().getBytes());
            //        return codat;
        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(JxtaNetworkModule.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(JxtaNetworkModule.class.getName()).log(Level.SEVERE, null, ex);
        }

        return null;
    }

    /**
     * Generate a new NodeADV ID for a custom peer of the group 'peerGroup'
     * 
     * @param version
     * @param peerGroup
     * @param peerID
     * @return
     */
    public static CodatID createNodeAdvertisementID(Long version, PeerGroup peerGroup, String peerID) {
        try {
            //Codice preso da PeerAdvertisement per creare l'ID del PeerAdvertisement stesso
            //TODO if necessary use the peer id instead
            String hashValue = version.toString() + NodeAdvertisement.getAdvertisementType() + peerGroup.getPeerGroupID().getUniqueValue().toString() + peerID;
            byte[] seed = hashValue.getBytes("UTF-8");
//            byte[] variant = version.toString().getBytes("UTF-8");

            return IDFactory.newCodatID(peerGroup.getPeerGroupID(), seed, new ByteArrayInputStream(seed));

        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(JxtaNetworkModule.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(JxtaNetworkModule.class.getName()).log(Level.SEVERE, null, ex);
        }

        return null;
    }

    /**
     * Create a PipeID for this peer
     * 
     * @param pg
     * @return
     */
    public static PipeID createPipeID(PeerGroup pg) {
        try {
            //Codice preso da PeerAdvertisement per creare l'ID del PeerAdvertisement stesso
            //TODO if necessary use the peer id instead
            String hashValue = PipeAdvertisement.getAdvertisementType() + pg.getPeerGroupID().getUniqueValue().toString() + pg.getPeerID().getUniqueValue().toString();
            byte[] seed = hashValue.getBytes("UTF-8");
//            //TODO non funziona da sempre lo stesso id
//            return IDFactory.newPipeID(p2pMapReduceGroup.getPeerGroupID(), seed);
            //FIX non si sa se ritorni un id univoco su tutta la rete
            return IDFactory.newPipeID(pg.getPeerGroupID());


        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(JxtaNetworkModule.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(JxtaNetworkModule.class.getName()).log(Level.SEVERE, null, ex);
        }

        return null;
    }
}
