/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.impl.networkModule.jxta;

import java.util.HashSet;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import p2pMapReduce.networkModule.NodeInfo;
import p2pMapReduce.nodeModule.NodeModuleIF;
import p2pMapReduce.nodeModule.event.EventType;
import p2pMapReduce.nodeModule.event.InnerEvent;

/**
 * Execute a periodic control of the local NodeAdvertisement cache.
 * If some NodeAdvertisement are expired a failure event is notified to
 * the associated NodeModule
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 * @deprecated no longer used
 */
public class NodeInfoController implements Runnable {

    private static final transient Logger LOG = Logger.getLogger(NodeInfoController.class.getName());
    private JxtaNetworkModule networkModule;
    //FIXME too expensive to keep in memory ?
    Set<NodeInfo> previousView;
    private final Object lock = new Object();

    public NodeInfoController(JxtaNetworkModule networkModule) {
        this.networkModule = networkModule;
        previousView = new HashSet<NodeInfo>();
    }

    void flush(NodeInfo failedNodeInfo) {
        synchronized (lock) {
            previousView.remove(failedNodeInfo);
        }
    }

    public void run() {
        while (true) {
            try {
                Thread.sleep(Math.min(JxtaNetworkModule.PERIODIC_FIND_TIMEOUT, JxtaNetworkModule.PERIODIC_PUBLISH_TIMEOUT));
                NodeModuleIF nodeModule = networkModule.getNodeModule();
                if (nodeModule == null) {
                    continue;
                }

                Set<NodeInfo> currentView = networkModule.getAllAvailableNodesFromLocalCache();
                synchronized (lock) {
                    for (NodeInfo node : previousView) {
                        //TODO need to check if it is an obsolete version?
                        if (!currentView.contains(node)) {
                            LOG.log(Level.INFO, "No more ADV available for {0}. Notifing node failure", node.getID());
                            InnerEvent failureEvent = new InnerEvent(EventType.NODE_FAILURE, node.getID().toString());
                            nodeModule.handleEvent(failureEvent);
                        }
                    }
                    previousView = currentView;
                }

            } catch (InterruptedException ex) {
                LOG.log(Level.SEVERE, null, ex);
            }
        }
    }
}
