/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.mapreduceModule.input;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import p2pMapReduce.mapreduceModule.TaskAttemptContext;

/**
 * Treats line number in the file as key and the line itself as value
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class LineRecordReader extends RecordReader<Long, String> {

    private static final int LINE_SEPARATOR_BYTES = TextFileInputSplit.LINE_SEPARATOR.getBytes().length;
    TextFileInputSplit tfInputSplit;
    BufferedReader br;
    Long key = new Long(0);
    String value;
    long currentBytesRead;

    @Override
    public void initialize(InputSplit split, TaskAttemptContext context) throws IOException, InterruptedException {
        tfInputSplit = (TextFileInputSplit) split;
        br = new BufferedReader(new FileReader(tfInputSplit.getFile()));
    }

    @Override
    public boolean nextKeyValue() throws IOException, InterruptedException {
        value = br.readLine();
        if (value != null) {
            //FIXME value.getBytes() can be time consuming ??
            currentBytesRead += value.getBytes().length + LINE_SEPARATOR_BYTES;
            key = new Long(key + 1);
            return true;
        } else {
            key = null;
            return false;
        }
    }

    @Override
    public Long getCurrentKey() throws IOException, InterruptedException {
        return key;
    }

    @Override
    public String getCurrentValue() throws IOException, InterruptedException {
        return value;
    }

    /**
     * Not correctly implemented yet
     * @return 
     * @throws IOException
     * @throws InterruptedException
     */
    @Override
    public float getProgress() throws IOException, InterruptedException {
        float progress = (float) (((double) currentBytesRead) / tfInputSplit.getLength());
        return Math.min(1F, progress);
    }

    @Override
    public void close() throws IOException {
        br.close();
    }
}
