/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.mapreduceModule.input;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataInput;
import java.io.DataOutput;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class TextFileInputSplit extends InputSplit {

    public static final String LINE_SEPARATOR = System.getProperty("line.separator");
    public static int LINE_SEPARATOR_BYTES = LINE_SEPARATOR.getBytes().length;
    private static final transient Logger LOG = Logger.getLogger(TextFileInputSplit.class.getName());
    private String LOG_PREFIX;
    private String filePath;
    private String inputFilePath;
    private File file;
    private long fileLength;
    private boolean inited;
    private PrintWriter pw;

    public TextFileInputSplit() {
    }

    public TextFileInputSplit(String filePath, String inputFilePath) throws FileNotFoundException, IOException {
        this.filePath = filePath;
        this.inputFilePath = inputFilePath;
        this.file = new File(filePath);
        LOG_PREFIX = "FileSplit for " + filePath + ": ";

    }

    public File getFile() {
        return file;
    }

    public String getInputFilePath() {
        return inputFilePath;
    }

    private void init() throws FileNotFoundException, IOException {
        file.getParentFile().mkdirs();
//        file.createNewFile();
        pw = new PrintWriter(new BufferedWriter(new FileWriter(file)));

        inited = true;
    }

    public long append(BufferedReader br, long bytesRemaining) throws FileNotFoundException, IOException {
        if (!inited) {
            init();
        }

        long bytesWritten = 0;
        while (bytesWritten < bytesRemaining) {
            String line = br.readLine();
            if (line == null) {
                LOG.log(Level.FINE, "File terminated");
                break;
            }
            pw.println(line);
            bytesWritten += line.getBytes().length + LINE_SEPARATOR_BYTES;
        }

        if (bytesWritten < bytesRemaining) {
            LOG.log(Level.FINE, "Premature end of file, bytes written: {0} while bytes to write were {1}", new Object[]{bytesWritten, bytesRemaining});
        }

        fileLength += bytesWritten;

        return bytesWritten;

    }

    public void close() throws IOException {
        pw.close();
        DataOutputStream dos2 = new DataOutputStream(new FileOutputStream(filePath + DESCRIPTOR_SUFFIX));
        write(dos2);
        dos2.close();
    }

    @Override
    public long getLength() throws IOException, InterruptedException {
        if (!inited) {
            return 0;
        } else {
            return fileLength;
        }
    }

    /**
     * @deprecated not supported
     * @return
     * @throws IOException
     * @throws InterruptedException
     */
    @Override
    public String[] getLocations() throws IOException, InterruptedException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public String toString() {
        return LOG_PREFIX;
    }

    public void write(DataOutput out) throws IOException {
        byte bytes[] = filePath.getBytes("UTF-8");
        out.writeInt(bytes.length);
        out.write(bytes, 0, bytes.length);
        bytes = inputFilePath.getBytes("UTF-8");
        out.writeInt(bytes.length);
        out.write(bytes, 0, bytes.length);
        out.writeLong(fileLength);
    }

    public void readFields(DataInput in) throws IOException {
        int length = in.readInt();
        byte bytes[] = new byte[length];
        in.readFully(bytes);
        filePath = new String(bytes, "UTF-8");
        length = in.readInt();
        bytes = new byte[length];
        in.readFully(bytes);
        inputFilePath = new String(bytes, "UTF-8");
        fileLength = in.readLong();
        file = new File(filePath);
        LOG_PREFIX = "FileSplit for " + filePath + ": ";

    }
}
