/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.mapreduceModule.input;

import java.io.BufferedReader;
import java.io.DataInput;
import java.io.DataOutput;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class TextFileInputSplit_old extends InputSplit {

    public static final String LINE_SEPARATOR = System.getProperty("line.separator");
    
    private static final transient Logger LOG = Logger.getLogger(TextFileInputSplit.class.getName());
    private String LOG_PREFIX;
    private String filePath;
    private String inputFilePath;
    private File file;
    private boolean inited;
    //TODO optimization: why not use a print writer??
    private DataOutputStream dos;

    public TextFileInputSplit_old() {
    }

    public TextFileInputSplit_old(String filePath, String inputFilePath) throws FileNotFoundException, IOException {
        this.filePath = filePath;
        this.inputFilePath = inputFilePath;
        this.file = new File(filePath);
        LOG_PREFIX = "FileSplit for " + filePath + ": ";

    }

    public File getFile() {
        return file;
    }

    public String getInputFilePath() {
        return inputFilePath;
    }

    private void init() throws FileNotFoundException, IOException {
        file.getParentFile().mkdirs();
//        file.createNewFile();
        dos = new DataOutputStream(new FileOutputStream(file));

        inited = true;
    }

    /**
     * Append the file content starting from source current position
     * on this file split
     * @param srcFile
     */
    @Deprecated
    public long appendFrom(BufferedReader br) throws FileNotFoundException, IOException {
        if (!inited) {
            init();
        }
        /*
         * By specification a file split must be at most 64*1024 bytes
         */
        long bytesWritten = 0;
        String line = br.readLine();
        while (line != null) {
            String toWrite = line + "\n";
            byte[] lineBytes = toWrite.getBytes();
            bytesWritten += lineBytes.length;
            line = br.readLine();
        }
        return bytesWritten;
    }

    /**
     * Appends the file content line by line until at least bytesRemaining
     * were written
     *
     * @param srcFile
     * @param bytesRemaining
     * @return the bytes effectively written
     * 
     */
    @Deprecated
    public long appendFrom(BufferedReader br, long bytesRemaining) throws FileNotFoundException, IOException {
        if (!inited) {
            init();
        }

        long bytesWritten = 0;
        while (bytesWritten < bytesRemaining) {
            String line = br.readLine();
            if (line == null) {
                break;
            }
            String toWrite = line + "\n";
            byte[] lineBytes = toWrite.getBytes();
            bytesWritten += lineBytes.length;
        }

        if (bytesWritten < bytesRemaining) {
            LOG.log(Level.FINE, "Premature end of file, bytes written: {0} while bytes to write were {1}", new Object[]{bytesWritten, bytesRemaining});
        }

        return bytesWritten;

    }

    public long append(BufferedReader br, long bytesRemaining) throws FileNotFoundException, IOException {
        if (!inited) {
            init();
        }

        long bytesWritten = 0;
        long startSize = dos.size();
        while (bytesWritten < bytesRemaining) {
            String line = br.readLine();
            if (line == null) {
                LOG.log(Level.FINE, "File terminated");
                break;
            }
            byte bytesToWrite[] = (line + LINE_SEPARATOR).getBytes();
            dos.write(bytesToWrite);
            bytesWritten += bytesToWrite.length;
        }

//        bytesWritten = dos.size() - startSize;

        if (bytesWritten < bytesRemaining) {
            LOG.log(Level.FINE, "Premature end of file, bytes written: {0} while bytes to write were {1}", new Object[]{bytesWritten, bytesRemaining});
        }

        return bytesWritten;

    }

    public void close() throws IOException {
        dos.close();
        DataOutputStream dos2 = new DataOutputStream(new FileOutputStream(filePath + DESCRIPTOR_SUFFIX));
        write(dos2);
        dos2.close();
    }

    @Override
    public long getLength() throws IOException, InterruptedException {
        if (!inited) {
            return 0;
        } else {
            return dos.size();
        }
    }

    @Override
    public String[] getLocations() throws IOException, InterruptedException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public String toString() {
        return LOG_PREFIX;
    }

    public void write(DataOutput out) throws IOException {
        byte bytes[] = filePath.getBytes("UTF-8");
        out.writeInt(bytes.length);
        out.write(bytes, 0, bytes.length);
        bytes = inputFilePath.getBytes("UTF-8");
        out.writeInt(bytes.length);
        out.write(bytes, 0, bytes.length);
    }

    public void readFields(DataInput in) throws IOException {
        int length = in.readInt();
        byte bytes[] = new byte[length];
        in.readFully(bytes);
        filePath = new String(bytes, "UTF-8");
        length = in.readInt();
        bytes = new byte[length];
        in.readFully(bytes);
        inputFilePath = new String(bytes, "UTF-8");

        file = new File(filePath);
        LOG_PREFIX = "FileSplit for " + filePath + ": ";

    }
}
