/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.mapreduceModule.io;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

/**
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class IntWritable implements WritableComparable<IntWritable> {

    private int value;

    public IntWritable() {
    }

    public IntWritable(int value) {
        this.value = value;
    }

    public long getValue() {
        return value;
    }

    public void setValue(int value) {
        this.value = value;
    }

    public void readFields(DataInput in) throws IOException {
        value = in.readInt();
    }

    public void write(DataOutput out) throws IOException {
        out.writeInt(value);
    }

    public int compareTo(IntWritable o) {
        if (value < o.value) {
            return -1;
        } else if (value > o.value) {
            return 1;
        } else {
            return 0;
        }
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final IntWritable other = (IntWritable) obj;
        if (this.value != other.value) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 47 * hash + this.value;
        return hash;
    }

    @Override
    public String toString() {
        return "" + value;
    }
}
