/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.mapreduceModule.io;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

/**
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class LongWritable implements WritableComparable<LongWritable> {

    private long value;

    public LongWritable() {
    }

    public LongWritable(long value) {
        this.value = value;
    }

    public long getValue() {
        return value;
    }

    public void setValue(long value) {
        this.value = value;
    }

    public void readFields(DataInput in) throws IOException {
        value = in.readLong();
    }

    public void write(DataOutput out) throws IOException {
        out.writeLong(value);
    }

    public int compareTo(LongWritable o) {
        if (value < o.value) {
            return -1;
        } else if (value > o.value) {
            return 1;
        } else {
            return 0;
        }
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final LongWritable other = (LongWritable) obj;
        if (this.value != other.value) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 5;
        hash = 29 * hash + (int) (this.value ^ (this.value >>> 32));
        return hash;
    }

    @Override
    public String toString() {
        return "" + value;
    }
}
