/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.mapreduceModule.io;

import java.io.DataInput;
import java.io.DataInputStream;
import java.io.DataOutput;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.charset.Charset;

/**
 * A writable that represent a string.
 * The string is always converted in UTF-8 encoding
 * 
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class StringWritable implements WritableComparable<StringWritable> {

    private static final Charset UTF8 = Charset.forName("UTF-8");
    private String value;

    public static void main(String[] args) throws IOException {
        File f = File.createTempFile("ciao", ".pp");
        FileOutputStream fos = new FileOutputStream(f);

        StringWritable sw = new StringWritable("aduahdi as \"s c2wu\\8273777\\7\\7\\7717777//////.htt871   48c 337473843o48474584ciao");

        sw.write(new DataOutputStream(fos));

        fos.close();

        FileInputStream fis = new FileInputStream(f);

        StringWritable sw2 = new StringWritable(null);
        sw2.readFields(new DataInputStream(fis));

        System.out.println(sw.equals(sw2));
    }

    public static void write(DataOutput out, String s) throws IOException {
        byte[] bytes = s.getBytes(UTF8);
        out.writeInt(bytes.length);
        out.write(bytes);
    }

    public static String read(DataInput in) throws IOException {
        int length = in.readInt();
        byte[] bytes = new byte[length];
        in.readFully(bytes, 0, length);
        return new String(bytes, UTF8);
    }

    public StringWritable() {
    }

    public StringWritable(String value) {
        setValue(value);
    }

    public String getValue() {
        return value;
    }

    public void setValue(String value) {
        this.value = new String(value.getBytes(UTF8));
    }

    public void readFields(DataInput in) throws IOException {
        int length = in.readInt();
        byte[] bytes = new byte[length];

        in.readFully(bytes, 0, length);

        value = new String(bytes, UTF8);
    }

    public void write(DataOutput out) throws IOException {
        byte[] bytes = value.getBytes(UTF8);
        out.writeInt(bytes.length);
        out.write(bytes);
    }

    public int compareTo(StringWritable o) {
        return value.compareTo(o.value);
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final StringWritable other = (StringWritable) obj;
        if ((this.value == null) ? (other.value != null) : !this.value.equals(other.value)) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 5;
        hash = 29 * hash + (this.value != null ? this.value.hashCode() : 0);
        return hash;
    }

    @Override
    public String toString() {
        return value;
    }
}
