/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.networkModule;

import java.io.File;
import java.util.HashMap;
import java.util.Random;
import net.jxta.platform.NetworkManager.ConfigMode;
import p2pMapReduce.impl.networkModule.jxta.JxtaNetworkModule;

/**
 * Contains all the startup and configuration parameters for a NetworkModuleIF.
 * Once it has been used to create a network module it should not be used more.
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
//TODO a NetworkConfiguration instance should be retrieved by the network module
public class NetworkConfiguration {

    /**
     * Boolean value
     * Indicates to the network module if this node is a user node.
     */
    public static final String USER_NODE = "user_node";
    /**
     * Value is a path that refers to a directory which the network module
     * consider as its home directory
     */
    private static final String NETWORK_MODULE_HOME_DIR = JxtaNetworkModule.JXTA_HOME_ATTR;
    /**
     * Map that contains all the configuration parameters
     */
    private HashMap<String, Object> paramMap;

    public NetworkConfiguration() {

        paramMap = new HashMap<String, Object>();
        setDefaults();

    }

    public Object set(String key, Object value) {
        return paramMap.put(key, value);
    }

    public Object get(String key) {
        return paramMap.get(key);
    }

    private void setDefaults() {
        //FIXME reference to JxtaNetworkModule must be removed
        String tmpDir = System.getProperty("java.io.tmpdir");
        paramMap.put(JxtaNetworkModule.JXTA_PEER_MODE_ATTR, ConfigMode.EDGE);
        paramMap.put(JxtaNetworkModule.IS_GATEWAY_ATTR,false);
        paramMap.put(JxtaNetworkModule.RDV_AUTO_START,false);
        paramMap.put(NETWORK_MODULE_HOME_DIR, createTempDir(tmpDir + "/jxta/"));
        paramMap.put(USER_NODE, false);
    }

    /**
     *
     * @return a random directory name
     */
    public static String createTempDir(String rootDir) {
        Random r = new Random();
        String dir = rootDir + "tmp_" + r.nextInt() + "/";
        File f = new File(dir);
        while (!f.mkdirs()) {
            dir = rootDir + "tmp_" + r.nextInt() + "/";
            f = new File(dir);
        }
        return dir;
    }
}
