/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.nodeModule;

import p2pMapReduce.networkModule.NodeInfo;
import java.util.HashMap;
import java.util.LinkedList;
import p2pMapReduce.fs.FileSystemView;
import p2pMapReduce.ftp.FTPManager;
import p2pMapReduce.nodeModule.event.EventListener;
import p2pMapReduce.networkModule.NetworkModuleIF;
import p2pMapReduce.entity.BackupJob;
import p2pMapReduce.entity.PrimaryJob;
import p2pMapReduce.entity.SlaveTask;
import p2pMapReduce.nodeModule.nodeFsm.NodeFSM;

/**
 * A node module contains a finite state machine ({@link NodeFSM}) that
 * implements the node behavior, and all the data required.
 * The data are represented by {@link PrimaryJob}, {@link BackupJob},
 * {@link SlaveTask}, {@link TaskManager }
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
//TODO needed to extends EventListener? for now it s not needed
public interface NodeModuleIF extends EventListener {

    /**
     * 
     * @return The {@link NetworkModuleIF} associated to this node
     */
    NetworkModuleIF getNetworkModule();

    /**
     * 
     * @return The {@link FTPManager} for this node
     */
    FTPManager getFtpManager();

    /**
     *
     * @return The {@link FileSystemView} of this node
     */
    FileSystemView getFileSystemView();

    /**
     *
     * @return {@link NodeInfo} instance associated to this node
     */
    NodeInfo getNodeInfo();

    /**
     *
     * @return The {@link NodeFSM} of this node
     */
    NodeFSM getNodeFSM();

    /**
     * 
     * 
     * @return The id of this node
     */
    String getNodeID();

    /**
     * The id of the network coordinator
     * @return
     */
    String getCoordinatorID();

    void setCoordinatorID(String coordinatorID);

    /**
     * Return true if this node is the Coordinator
     * @return
     */
    boolean isCoordinator();

    LinkedList<PrimaryJob> getPrimaryJobList();

    void addPrimaryJob(PrimaryJob primaryJob);

    PrimaryJob removePrimaryJob(String jobID);

    PrimaryJob getPrimaryJob(String jobID);

    LinkedList<BackupJob> getBackupJobList();

    void addBackupJob(BackupJob primaryJob);

    BackupJob removeBackupJob(String jobID);

    BackupJob getBackupJob(String jobID);

    HashMap<String, SlaveTask> getSlaveTaskMap();

    void addSlaveTask(SlaveTask st);

    SlaveTask getSlaveTask(String taskID);

    SlaveTask removeSlaveTask(String taskID);

    //TODO refactoring: mantenere solo i task manager e non i slave task
    void addTaskManager(String taskID, TaskManager tm);

    TaskManager getTaskManager(String taskID);

    TaskManager removeTaskManager(String taskID);
}
