/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.nodeModule;

import java.io.IOException;
import java.util.Properties;
import p2pMapReduce.networkModule.NetworkConfiguration;
import p2pMapReduce.impl.networkModule.jxta.JxtaNetworkModule;
import p2pMapReduce.impl.nodeModule.NodeModuleImpl;
import p2pMapReduce.gui.GUIFrame;
import p2pMapReduce.gui.LogPanel;
import p2pMapReduce.util.P2PMapReduceConfig;
import p2pMapReduce.util.LoggingUtil;

/**
 * Main class to start a P2PMapReduce node
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class NodeModuleStarter {

    public static void main(String[] args) {
        try {
            startNode();

        } catch (Throwable t) {
            t.printStackTrace();
        }
    }

    public static void startNode() throws IOException {
        Properties configParams = P2PMapReduceConfig.getConfiguration();
        LoggingUtil.setLogging();

        NetworkConfiguration nc = new NetworkConfiguration();

        boolean isGateway = Boolean.parseBoolean(configParams.getProperty(JxtaNetworkModule.IS_GATEWAY_ATTR, nc.get(JxtaNetworkModule.IS_GATEWAY_ATTR).toString()));
        String rdvSeeds = configParams.getProperty(JxtaNetworkModule.RDV_SEEDS_ATTR);
        boolean rdvAutoStart = Boolean.parseBoolean(configParams.getProperty(JxtaNetworkModule.RDV_AUTO_START, nc.get(JxtaNetworkModule.RDV_AUTO_START).toString()));
        boolean gui = Boolean.parseBoolean(configParams.getProperty(P2PMapReduceConfig.GUI_ATTR));
        boolean redirect = Boolean.parseBoolean(configParams.getProperty(P2PMapReduceConfig.GUI_OUTPUT_REDIRECT));
        boolean defaultOut = Boolean.parseBoolean(configParams.getProperty(P2PMapReduceConfig.GUI_OUTPUT_DEFAULT));
        GUIFrame guiFrame = null;
        if (gui) {
            LogPanel jtos = new LogPanel(redirect, defaultOut);
            guiFrame = new GUIFrame(jtos);
        }

        nc.set(JxtaNetworkModule.IS_GATEWAY_ATTR, isGateway);
        nc.set(JxtaNetworkModule.RDV_AUTO_START, rdvAutoStart);
        if (rdvSeeds != null) {
            String[] rdvSeedsArr = rdvSeeds.split(",");
            nc.set(JxtaNetworkModule.RDV_SEEDS_ATTR, rdvSeedsArr);
        } else {
            nc.set(JxtaNetworkModule.RDV_SEEDS_ATTR, null);
        }
        NodeModuleImpl nodeModule = (NodeModuleImpl) NodeModuleFactory.createNodeModule(nc);
        if (gui) {
            guiFrame.setNodeModule(nodeModule);
        }
    }
}
