/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.nodeModule.event;

import p2pMapReduce.nodeModule.nodeFsm.NodeFSM;

/**
 * An abstract representation of P2PMapReduce framework event.
 * An event can be an {@link InnerEvent} generated by the {@link NodeFSM}
 * of this node, or a {@link MessageEvent} received from a remote node.
 *
 * Each event is characterized by:
 *  - a type specified by an {@link EventType}
 *  - an array of event parameters. The type and order of parameters
 *    strictly depends on the event type
 * 
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public interface EventIF {

    EventType getType();

    Object getParam(int index);

    Object[] getParams();

    void setParams(Object... params);

    void setType(EventType type);
}
