/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.nodeModule.event;

import java.util.HashSet;
import java.util.Set;

/**
 * An utility class to notify en event to a set of {@link EventListener}
 * 
 * @author Francesco De Luca f.deluca86@gmail.com
 */
/*
 * To avoid deadlock problem with JXTABidiPipeConnection takes care
 * that the notification is asinchronous, i.e. the notifyEvent
 * mehtod must be call in a Thread different from that which has created
 * or received the event
 */
public class EventNotifier {

    private Set<EventListener> eventListenerSet;

    public EventNotifier() {

        eventListenerSet = new HashSet<EventListener>();
    }

    public EventNotifier(Set<EventListener> eventListenerSet) {
        this.eventListenerSet = eventListenerSet;
    }

    public synchronized void addEventListener(EventListener eventListener) {
        eventListenerSet.add(eventListener);
    }

    public synchronized boolean removeEventListener(EventListener eventListener) {
        return eventListenerSet.remove(eventListener);
    }

    /**
     * Used to notify the event to all the event listener of the set
     * @param event
     */
    public synchronized void notifyEvent(EventIF event) {
        for (EventListener eventListener : eventListenerSet) {
            eventListener.handleEvent(event);
        }
    }
}
