/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.nodeModule.event;

import p2pMapReduce.util.LoggingUtil;

/**
 * An inner event that cannot be sent across the network.
 * Only internal usage.
 * 
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class InnerEvent implements EventIF {

    private EventType type;
    private Object[] params;

    public InnerEvent(EventType type) {
        this.type = type;
        params = new Object[0];
    }

    public InnerEvent(EventType type, Object... params) {
        this.type = type;
        this.params = params;
    }

    public EventType getType() {
        return type;
    }

    public void setType(EventType type) {
        this.type = type;
    }

    public Object getParam(int index) {
        if (index < 0 || index >= params.length) {
            throw new ArrayIndexOutOfBoundsException("Params " + params.length + " index " + index);
        }

        return params[index];
    }

    public Object[] getParams() {
        return params;
    }

    public void setParams(Object... params) {
        this.params = params;
    }

    @Override
    public String toString() {

        String s = "IE [Type=" + type.toString() + " Params: ";
        for (Object o : getParams()) {
            if (o instanceof Iterable) {
                s += "\n" + LoggingUtil.elementByLine((Iterable) o) + "\n";
            } else {
                s += o.toString() + " ";
            }
        }
        return s + "]";
    }
}
