/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.nodeModule.event;

import java.io.Serializable;
import p2pMapReduce.util.LoggingUtil;

/**
 * A serializable implementation of the Event interface.
 * This event can also be sent to remote nodes across a network, so
 * it's required that all parameters are serializable object.
 * 
 * @author Francesco De Luca f.deluca86@gmail.com
 *
 */
// NOTE: Directly implements the EventIF interface for problem with the
// serialization of enum variable 'type' --> the problem was that super type
// serialization must be directly handled by sub type class
//TODO optimization: implements Externalizable
public class MessageEvent implements EventIF, Serializable {

    private EventType type;
    private Serializable[] params;

    public MessageEvent(EventType type) {
        this.type = type;
        params = new Serializable[0];
    }

    public MessageEvent(EventType type, Serializable... params) {
        this.type = type;
        this.params = params;
    }

    public Object getParam(int index) {
        return params[index];
    }

    public Serializable[] getParams() {
        return params;
    }

    public void setParams(Object... params) {
        this.params = new Serializable[params.length];
        int i = 0;
        for (Object param : params) {
            if (!(param instanceof Serializable)) {
                this.params = null;
                throw new IllegalArgumentException("Try to insert no serializable param in a MessageEvent: " + param);
            } else {
                this.params[i] = (Serializable) param;
                i++;
            }
        }
    }

    public EventType getType() {
        return type;
    }

    public void setType(EventType type) {
        this.type = type;
    }

    @Override
    public String toString() {
        String s = "ME [Type=" + type.toString() + " Params: ";
        for (Object o : getParams()) {
            if (o instanceof Iterable) {
                s += "\n" + LoggingUtil.elementByLine((Iterable) o) + "\n";
            } else {
                s += o.toString() + " ";
            }
        }
        return s + "]";
    }
}
