/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.nodeModule.nodeFsm;

import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import p2pMapReduce.networkModule.NetworkModuleIF;
import p2pMapReduce.networkModule.NodeInfo;

/**
 * Inner state of Slave macro state.
 * A slave in the {@link Idle} state periodically transit to this
 * state to check for the existence of one master in the network.
 * If no masters are founded the node automatically become a master by
 * transit to {@link Master} macro state.
 * 
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class Check_Master extends NodeState {

    private static final Logger LOG = Logger.getLogger(Check_Master.class.getName());
    //TODO move into NodeState
    NodeFSM nodeFSM;

    public Check_Master(NodeFSM aThis) {
        this.nodeFSM = aThis;
    }

    @Override
    protected synchronized void entryAction() {
        LOG.log(Level.INFO, "entryAction");
        NetworkModuleIF netModule = nodeFSM.getNodeModule().getNetworkModule();
        Set<NodeInfo> masters = netModule.getAllMasters();
        if (masters.isEmpty()) {
            LOG.log(Level.INFO, "Transition to MASTER");
            nodeFSM.transition(nodeFSM.MASTER);
        } else {
            LOG.log(Level.INFO, "Transition to IDLE");
            nodeFSM.transition(nodeFSM.IDLE);
        }

    }

}
