/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.nodeModule.nodeFsm;



/**

 * Base class for a finite state machine.

 * Each finite state machine has a current state.

 * The only function is the transition from the current state to the new current

 * state.

 * 

 * @author Francesco De Luca f.deluca86@gmail.com

 */

public class FSM {



    private State currentState;



    /**

     * Make the FSM to transit to {@code nextState}

     * Calls {@link State#exitAction()} on the current state, update the current

     * state and calls on it {@link State#entryAction()}

     * 

     * @param nextState

     */

    public void transition(State nextState) {

        if (currentState != null) {

            currentState.exitAction();

        }

        currentState = nextState;

        currentState.entryAction();

    }//transition



    public State currentState() {

        return currentState;

    }

}//FSM



