/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.util;

import java.io.File;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectStreamClass;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.Socket;
import java.net.URL;
import java.net.URLClassLoader;
import java.net.URLDecoder;
import java.net.UnknownHostException;
import java.util.Enumeration;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class ClassLoaderUtil {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
//        customURLClassLoader();
        String path = findContainingJar(ClassLoaderUtil.class);
        System.out.println("path = " + path);
    }

    /**
     * Used to add some URL resource to the system class loader
     * 
     * @param toLoad
     * @throws Exception
     */
    public static void addToSystemClassLoader(URL... toLoad) throws Exception {
        //TODO after job execution the relative URLs must be removed
        //    from class loader
        ClassLoader cl = ClassLoader.getSystemClassLoader();
//        System.out.println("cl = " + cl);
//        System.out.println("cl parent " + cl.getParent());
        URLClassLoader sysloader = (URLClassLoader) ClassLoader.getSystemClassLoader();
        Class sysclass = URLClassLoader.class;
        Method method = sysclass.getDeclaredMethod("addURL", URL.class);
        method.setAccessible(true);
        for (URL url : toLoad) {

            method.invoke(sysloader, new Object[]{url});

        }

    }

    /**
     * From hadoop
     * @param my_class
     * @return
     */
    private static String findContainingJar(Class my_class) {
        ClassLoader loader = my_class.getClassLoader();
        String class_file = my_class.getName().replaceAll("\\.", "/") + ".class";
        try {
            for (Enumeration itr = loader.getResources(class_file); itr.hasMoreElements();) {
                URL url = (URL) itr.nextElement();
                if ("jar".equals(url.getProtocol())) {
                    String toReturn = url.getPath();
                    if (toReturn.startsWith("file:")) {
                        toReturn = toReturn.substring("file:".length());
                    }
                    toReturn = URLDecoder.decode(toReturn, "UTF-8");
                    return toReturn.replaceAll("!.*$", "");
                }
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return null;
    }

    public static void customURLClassLoader() {
        try {
            File jar = new File("/home/francesco/Scrivania/Collegamento a NetBeansProjects/ClassSender/dist/ClassSender.jar");
            URL jarURL = jar.toURI().toURL();

            ClassLoader cl = ClassLoader.getSystemClassLoader();
            System.out.println("cl = " + cl);
            System.out.println("cl parent " + cl.getParent());

            final URLClassLoader ucl = new URLClassLoader(new URL[]{jarURL});

            System.out.println("ucl parent " + ucl.getParent());

            Socket s = new Socket("localhost", 9999);
            ObjectInputStream ois = new ObjectInputStream(s.getInputStream()) {

                @Override
                protected Class<?> resolveClass(ObjectStreamClass desc) throws IOException, ClassNotFoundException {
                    try {
                        String name = desc.getName();
                        return Class.forName(name, true, ucl);
                    } catch (ClassNotFoundException e) {
                        return super.resolveClass(desc);
                    }
                }
            };


            String className = (String) ois.readObject();
            System.out.println("className = " + className);

            Class loaded = ucl.loadClass(className);
            System.out.println("loaded = " + loaded);


            Class c = (Class) ois.readObject();
            ois.close();
            System.out.println("c = " + c);

            Object o = c.getConstructor().newInstance();
            System.out.println("o = " + o);

            Runnable r = (Runnable) o;
            System.out.println("r = " + r);

            Class c2 = Class.forName("classsender.test.TestClass2");
            System.out.println("c2 = " + c2);

        } catch (InstantiationException ex) {
            Logger.getLogger(ClassLoaderUtil.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            Logger.getLogger(ClassLoaderUtil.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalArgumentException ex) {
            Logger.getLogger(ClassLoaderUtil.class.getName()).log(Level.SEVERE, null, ex);
        } catch (InvocationTargetException ex) {
            Logger.getLogger(ClassLoaderUtil.class.getName()).log(Level.SEVERE, null, ex);
        } catch (ClassNotFoundException ex) {
            Logger.getLogger(ClassLoaderUtil.class.getName()).log(Level.SEVERE, null, ex);
        } catch (UnknownHostException ex) {
            Logger.getLogger(ClassLoaderUtil.class.getName()).log(Level.SEVERE, null, ex);
        } catch (MalformedURLException ex) {
            Logger.getLogger(ClassLoaderUtil.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(ClassLoaderUtil.class.getName()).log(Level.SEVERE, null, ex);
        } catch (NoSuchMethodException ex) {
            Logger.getLogger(ClassLoaderUtil.class.getName()).log(Level.SEVERE, null, ex);
        } catch (SecurityException ex) {
            Logger.getLogger(ClassLoaderUtil.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
}
