/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.util;

import java.util.Map;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Some logging utility
 * 
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class LoggingUtil {

    public static void setLogging() {

        Properties p = P2PMapReduceConfig.getConfiguration();
        String loggingLevel = p.getProperty("logging.level", "WARNING");
        String JXTAloggingLevel = p.getProperty("net.jxta.logging.Logging", "SEVERE");
        setLogging(loggingLevel, JXTAloggingLevel);

    }

    public static void setLogging(String loggingLevel, String JXTALoggingLevel) {


        Logger.getLogger("").setLevel(Level.parse(loggingLevel));

        System.setProperty("net.jxta.logging.Logging", JXTALoggingLevel);
        System.setProperty("net.jxta.level", JXTALoggingLevel);

    }

    public static String elementByLine(Iterable iterable) {
        StringBuilder sb = new StringBuilder();
        for (Object object : iterable) {
            sb.append(object);
            sb.append("\n");
        }
        return sb.toString();
    }

    public static <K, V> StringBuilder elementByLine(Map<K, V> map) {
        StringBuilder sb = new StringBuilder();
        for (Map.Entry<K, V> entry : map.entrySet()) {
            sb.append(entry.getKey());
            if (entry.getValue() instanceof Iterable) {
                Iterable value = (Iterable) entry.getValue();
                int count = 0;
                StringBuilder sb2 = new StringBuilder();
                for (Object object : value) {
                    count++;
                    sb2.append("\t").append(object).append("\n");
                }
                sb.append(":").append(count).append("\n");
                sb.append(sb2);
            } else {
                sb.append(" ").append(entry.getValue()).append("\n");
            }
        }
        return sb;
    }

    public static String listStackTrace(StackTraceElement[] stackTrace) {
        StringBuilder sb = new StringBuilder();
        for (StackTraceElement stackTraceElement : stackTrace) {
            sb.append(stackTraceElement.toString());
            sb.append("\n");
        }
        return sb.toString();
    }

   
}
