/*
 * P2P-MapReduce
 *
 * Copyright (C) 2011 University of Calabria
 * Dept. of Electronics, Computer Science and Systems
 */
package p2pMapReduce.util;

import java.util.HashSet;

/**
 * Utility to keep object references in order
 * to make them not removable from garbage collector.
 * Useful for singleton instance.
 * 
 * @author Francesco De Luca f.deluca86@gmail.com
 */
public class ReferenceHolder extends Thread {

    private HashSet s = new HashSet();
    private static ReferenceHolder uniqueInstance = null;

    private ReferenceHolder() {
    }

    public static ReferenceHolder getInstance() {
        if (uniqueInstance == null) {
            uniqueInstance = new ReferenceHolder();
            // Auto persistenza
            uniqueInstance.persist(uniqueInstance);
            uniqueInstance.start();
        }
        return uniqueInstance;
    }

    @Override
    public synchronized void run() {
        try {

            while (true) {
                wait();
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (IllegalMonitorStateException imse) {
            imse.printStackTrace();
        }
    }

    public synchronized void persist(Object o) {
        s.add(o);
    }

    public synchronized void unPersist(Object o) {
        s.remove(o);
    }
}
